#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import tabs

from openstack_dashboard import api
from openstack_dashboard.dashboards.project.instances import tables as itables

from gbpui import client
from gbpui import column_filters as gfilters

import tables

import time

PTGsTable = tables.PTGsTable
External_PTGsTable = tables.ExternalPTGsTable


class PTGsTab(tabs.TableTab):
    table_classes = (PTGsTable,)
    name = _("Internal")
    slug = "policytargets"
    template_name = "horizon/common/_detail_table.html"

    def get_policy_targetstable_data(self):
        policy_targets = []
        try:
            policy_targets = client.policy_target_list(self.tab_group.request,
            tenant_id=self.tab_group.request.user.tenant_id)
            a = lambda x, y: gfilters.update_policy_target_attributes(x, y)
            policy_targets = [a(self.request, item) for item in policy_targets]
        except Exception as e:
            msg = _('Unable to retrieve policy_target list. %s') % (str(e))
            exceptions.handle(self.tab_group.request, msg)
            for policy_target in policy_targets:
                policy_target.set_id_as_name_if_empty()
        return policy_targets


class ExternalPTGsTab(tabs.TableTab):
    table_classes = (External_PTGsTable,)
    name = _("External")
    slug = "externalpolicytargets"
    template_name = "horizon/common/_detail_table.html"

    def get_external_policy_targetstable_data(self):
        external_policy_targets = []
        try:
            external_policy_targets = client.ext_policy_target_list(
                self.tab_group.request,
                tenant_id=self.tab_group.request.user.tenant_id)
            a = lambda x, y: gfilters.update_policy_target_attributes(x, y)
            external_policy_targets = [a(self.request, item)
                for item in external_policy_targets]
        except Exception as e:
            msg = _('Unable to retrieve policy_target list. %s') % (str(e))
            exceptions.handle(self.tab_group.request, msg)
            for policy_target in external_policy_targets:
                policy_target.set_id_as_name_if_empty()
        return external_policy_targets


class PTGTabs(tabs.TabGroup):
    slug = "policy_targettabs"
    tabs = (PTGsTab, ExternalPTGsTab)
    sticky = True


class PTGDetailsTab(tabs.Tab):
    name = _("Group Details")
    slug = "policy_targetdetails"
    template_name = "project/policytargets/_policy_target_details.html"
    failure_url = reverse_lazy('horizon:project:policy_target_group:index')

    def get_context_data(self, request):
        policy_targetid = self.tab_group.kwargs['policy_target_id']
        nsp = ''
        try:
            policy_target = client.policy_target_get(request, policy_targetid)
            l2_policy = client.l2policy_get(request,
                            policy_target["l2_policy_id"])
            l3_policy = client.l3policy_get(request,
                            l2_policy["l3_policy_id"])
            if policy_target['network_service_policy_id']:
                nsp_id = policy_target['network_service_policy_id']
                nsp = client.get_networkservice_policy(request, nsp_id)
        except Exception:
            exceptions.handle(
                request, _('Unable to retrieve group details.'),
                redirect=self.failure_url)
        return {'policy_target': policy_target,
                'l3_policy': l3_policy,
                'l2_policy': l2_policy,
                'nsp': nsp}


class PTGDetailsTabs(tabs.TabGroup):
    slug = "policy_targettabs"
    tabs = (PTGDetailsTab,)


class InstancesTab(tabs.TableTab):
    name = _("Members")
    slug = "members_tab"
    table_classes = (tables.InstancesTable,)
    template_name = ("horizon/common/_detail_table.html")
    preload = True

    def get_instances_data(self):
        policy_targetid = self.tab_group.kwargs['policy_target_id']
        filtered_instances = []
        try:
            policytargets = client.pt_list(self.request,
            tenant_id=self.request.user.tenant_id,
            policy_target_group_id=policy_targetid)
            policy_target_ports = [x.port_id for x in policytargets]
            marker = self.request.GET.get(
                tables.InstancesTable._meta.pagination_param, None)
            # TODO(Sumit): Setting paginate to False is a temporary
            # fix. Earlier, when paginate was set to True we were
            # retrieving instances in pages and were only processing
            # the first page. While pagination is required for
            # scaling to a large number of instances, we need to first
            # retrieve the instances in pages, then process them,
            # and then show the filtered list (filtered_instances)
            # in pages.
            instances, self._has_more = api.nova.server_list(
                self.request, search_opts={'marker': marker,
                                           'paginate': False})
            self._has_more = False
            instances = [item for item in instances
                    if not itables.is_deleting(item)]
            # port_list for a given instance is not available
            # when it query immediately after member create,
            # So adding half sec sleep before port list query
            if policy_target_ports:
                time.sleep(0.5)
            for item in instances:
                for port in api.neutron.port_list(self.request,
                                                  device_id=item.id):
                    if port.id in policy_target_ports:
                        filtered_instances.append(item)
                        break
        except Exception:
            self._has_more = False
            error_message = _('Unable to get instances')
            exceptions.handle(self.request, error_message)
            filtered_instances = []
        return filtered_instances


class ConsumedTab(tabs.TableTab):
    name = _('Consumed Policy Rule Set')
    slug = 'consumed_policy_rule_sets_tab'
    table_classes = (tables.ConsumedContractsTable,)
    template_name = ("horizon/common/_detail_table.html")

    def get_consumed_policy_rule_sets_data(self):
        try:
            policy_targetid = self.tab_group.kwargs['policy_target_id']
            policy_target = client.policy_target_get(
                self.request, policy_targetid)
            consumed_policy_rule_set_ids = policy_target.get(
                'consumed_policy_rule_sets')
            consumed_policy_rule_sets = []
            for _id in consumed_policy_rule_set_ids:
                consumed_policy_rule_sets.append(
                    client.policy_rule_set_get(self.request, _id))
            consumed_policy_rule_sets = [gfilters.update_pruleset_attributes(
                self.request, item) for item in consumed_policy_rule_sets]
            return consumed_policy_rule_sets
        except Exception:
            error_message = _('Unable to get consumed rule sets')
            exceptions.handle(self.request, error_message)
            return []


class ProvidedTab(tabs.TableTab):
    name = _('Provided Policy Rule Set')
    slug = 'provided_policy_rule_sets_tab'
    table_classes = (tables.ProvidedContractsTable,)
    template_name = ("horizon/common/_detail_table.html")

    def get_provided_policy_rule_sets_data(self):
        try:
            policy_targetid = self.tab_group.kwargs['policy_target_id']
            policy_target = client.policy_target_get(
                self.request, policy_targetid)
            provided_policy_rule_set_ids = policy_target.get(
                'provided_policy_rule_sets')
            provided_policy_rule_sets = []
            for _id in provided_policy_rule_set_ids:
                provided_policy_rule_sets.append(
                    client.policy_rule_set_get(self.request, _id))
            provided_policy_rule_sets = [gfilters.update_pruleset_attributes(
                self.request, item) for item in provided_policy_rule_sets]
            return provided_policy_rule_sets
        except Exception:
            error_message = _('Unable to get provided rule sets')
            exceptions.handle(self.request, error_message)
            return []


class PTGMemberTabs(tabs.TabGroup):
    slug = 'member_tabs'
    tabs = (InstancesTab, ProvidedTab, ConsumedTab, PTGDetailsTab,)
    stiky = True


class ExtProvidedTab(ProvidedTab):
    table_classes = (tables.ExtProvidedContractsTable,)

    def get_provided_policy_rule_sets_data(self):
        try:
            ext_policy_targetid = self.tab_group.kwargs['ext_policy_target_id']
            ext_policy_target = client.ext_policy_target_get(
                self.request, ext_policy_targetid)
            provided_policy_rule_set_ids = ext_policy_target.get(
                'provided_policy_rule_sets')
            provided_policy_rule_sets = []
            for _id in provided_policy_rule_set_ids:
                provided_policy_rule_sets.append(
                    client.policy_rule_set_get(self.request, _id))
            provided_policy_rule_sets = [gfilters.update_pruleset_attributes(
                self.request, item) for item in provided_policy_rule_sets]
            return provided_policy_rule_sets
        except Exception:
            error_message = _('Unable to get provided rule sets')
            exceptions.handle(self.request, error_message)
            return []


class ExtConsumedTab(ConsumedTab):
    table_classes = (tables.ExtConsumedContractsTable,)

    def get_consumed_policy_rule_sets_data(self):
        try:
            ext_policy_targetid = self.tab_group.kwargs['ext_policy_target_id']
            ext_policy_target = client.ext_policy_target_get(
                self.request, ext_policy_targetid)
            consumed_policy_rule_set_ids = ext_policy_target.get(
                'consumed_policy_rule_sets')
            consumed_policy_rule_sets = []
            for _id in consumed_policy_rule_set_ids:
                consumed_policy_rule_sets.append(
                    client.policy_rule_set_get(self.request, _id))
            consumed_policy_rule_sets = [gfilters.update_pruleset_attributes(
                self.request, item) for item in consumed_policy_rule_sets]
            return consumed_policy_rule_sets
        except Exception:
            error_message = _('Unable to get consumed rule sets')
            exceptions.handle(self.request, error_message)
            return []


class ExternalPTGMemberTabs(tabs.TabGroup):
    slug = 'members'
    tabs = (ExtProvidedTab, ExtConsumedTab)
    sticky = True



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import re

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages
from horizon import tabs
from horizon import workflows

from gbpui import client

import forms as policy_rule_set_forms
import tabs as policy_rule_set_tabs
import workflows as policy_rule_set_workflows

PolicyRuleSetTabs = policy_rule_set_tabs.ApplicationPoliciesTabs
PolicyRuleSetDetailsTabs = policy_rule_set_tabs.PolicyRuleSetDetailsTabs
PolicyRuleDetailsTabs = policy_rule_set_tabs.PolicyRuleDetailsTabs
PolicyClassifierDetailsTabs = policy_rule_set_tabs.PolicyClassifierDetailsTabs
PolicyActionDetailsTabs = policy_rule_set_tabs.PolicyActionDetailsTabs
AddPolicyRuleSet = policy_rule_set_workflows.AddContract
AddPolicyRule = policy_rule_set_workflows.AddPolicyRule
AddPolicyClassifier = policy_rule_set_workflows.AddPolicyClassifier


class IndexView(tabs.TabView):
    tab_group_class = (PolicyRuleSetTabs)
    template_name = 'project/application_policy/details_tabs.html'

    def post(self, request, *args, **kwargs):
        obj_ids = request.POST.getlist('object_ids')
        action = request.POST['action']
        obj_type = re.search('delete([a-z]+)', action).group(1)
        if not obj_ids:
            obj_ids.append(re.search('([0-9a-z-]+)$', action).group(1))
        if obj_type == 'policyaction':
            for obj_id in obj_ids:
                try:
                    client.policyaction_delete(request, obj_id)
                    messages.success(request, _('Deleted action %s') % obj_id)
                except Exception as e:
                    msg = _('Unable to delete action. %s') % (str(e))
                    exceptions.handle(request, msg)
        if obj_type == 'policyclassifier':
            for obj_id in obj_ids:
                try:
                    client.policyclassifier_delete(request, obj_id)
                    messages.success(
                        request, _('Deleted classifer %s') % obj_id)
                except Exception as e:
                    msg = _('Unable to delete action. %s') % (str(e))
                    exceptions.handle(request, msg)
        if obj_type == 'policyrule':
            for obj_id in obj_ids:
                try:
                    client.policyrule_delete(request, obj_id)
                    messages.success(request,
                                     _('Deleted rule %s') % obj_id)
                except Exception as e:
                    msg = _('Unable to delete action. %s') % (str(e))
                    exceptions.handle(request, msg)
        if obj_type == 'policyruleset':
            for obj_id in obj_ids:
                try:
                    client.policy_rule_set_delete(request, obj_id)
                    messages.success(request,
                                     _('Deleted rule %s') % obj_id)
                except Exception as e:
                    msg = _('Unable to delete action. %s') % (str(e))
                    exceptions.handle(request, msg)

        return self.get(request, *args, **kwargs)


class AddPolicyRuleSetView(workflows.WorkflowView):
    workflow_class = AddPolicyRuleSet
    template_name = "project/application_policy/addpolicy_rule_set.html"

    def get_object_id(self, policy_rule_set):
        return [policy_rule_set.id]


class UpdatePolicyRuleSetView(forms.ModalFormView):
    form_class = policy_rule_set_forms.UpdatePolicyRuleSetForm
    template_name = 'project/application_policy/update_policy_rule_set.html'

    def get_context_data(self, **kwargs):
        context = super(
            UpdatePolicyRuleSetView, self).get_context_data(**kwargs)
        context['policy_rule_set_id'] = self.kwargs['policy_rule_set_id']
        return context

    def get_initial(self):
        return {'policy_rule_set_id': self.kwargs['policy_rule_set_id']}


class AddPolicyRuleView(workflows.WorkflowView):
    workflow_class = AddPolicyRule
    template_name = "project/application_policy/addpolicyrule.html"

    def get_object_id(self, rule):
        return [rule.id]


class AddPolicyClassifierView(forms.ModalFormView):
    form_class = policy_rule_set_forms.AddPolicyClassifierForm
    template_name = "project/application_policy/add_policy_classifier.html"

    def get_success_url(self):
        return reverse('horizon:project:application_policy:index')

    def get_object_id(self, classifier):
        return [classifier.id]


class AddPolicyActionView(forms.ModalFormView):
    form_class = policy_rule_set_forms.AddPolicyActionForm
    template_name = "project/application_policy/add_policy_action.html"

    def get_success_url(self):
        return reverse('horizon:project:application_policy:index')

    def get_object_id(self, policyaction):
        return [policyaction.id]


class UpdatePolicyActionView(forms.ModalFormView):
    form_class = policy_rule_set_forms.UpdatePolicyActionForm
    template_name = "project/application_policy/update_policy_action.html"

    def get_context_data(self, **kwargs):
        context = super(
            UpdatePolicyActionView, self).get_context_data(**kwargs)
        context['policyaction_id'] = self.kwargs['policyaction_id']
        return context

    def get_initial(self):
        return {'policyaction_id': self.kwargs['policyaction_id']}


class PolicyRuleSetDetailsView(tabs.TabView):
    tab_group_class = (PolicyRuleSetDetailsTabs)
    template_name = 'project/application_policy/details_tabs.html'


class PolicyRuleDetailsView(tabs.TabView):
    tab_group_class = (PolicyRuleDetailsTabs)
    template_name = 'project/application_policy/details_tabs.html'


class UpdatePolicyRuleView(forms.ModalFormView):
    form_class = policy_rule_set_forms.UpdatePolicyRuleForm
    template_name = "project/application_policy/update_policy_rule.html"

    def get_context_data(self, **kwargs):
        context = super(UpdatePolicyRuleView, self).get_context_data(**kwargs)
        context['policyrule_id'] = self.kwargs['policyrule_id']
        return context

    def get_initial(self):
        return {'policyrule_id': self.kwargs['policyrule_id']}


class PolicyClassifierDetailsView(tabs.TabView):
    tab_group_class = (PolicyClassifierDetailsTabs)
    template_name = 'project/application_policy/details_tabs.html'


class UpdatePolicyClassifierView(forms.ModalFormView):
    form_class = policy_rule_set_forms.UpdatePolicyClassifierForm
    template_name = "project/application_policy/update_policy_classifier.html"

    def get_context_data(self, **kwargs):
        context = super(
            UpdatePolicyClassifierView, self).get_context_data(**kwargs)
        context['policyclassifier_id'] = self.kwargs['policyclassifier_id']
        return context

    def get_initial(self):
        return {'policyclassifier_id': self.kwargs['policyclassifier_id']}


class PolicyActionDetailsView(tabs.TabView):
    tab_group_class = (PolicyActionDetailsTabs)
    template_name = 'project/application_policy/details_tabs.html'



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from django.core.urlresolvers import reverse
from django import http
from django import shortcuts
from django.utils import html
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages
from openstack_dashboard import api

from gbpui import client
from gbpui import fields

LOG = logging.getLogger(__name__)

EXT_SEG_PARAM_URL = "horizon:project:network_policy:add_external_segment_param"
NETWORK_PARAM_URL = "horizon:project:network_policy:add_network_service_param"
ROUTE_URL = "horizon:project:network_policy:add_external_route_param"


class BaseUpdateForm(forms.SelfHandlingForm):

    def clean(self):
        cleaned_data = super(BaseUpdateForm, self).clean()
        updated_data = {d: cleaned_data[d] for d in cleaned_data
            if d in self.changed_data}
        return updated_data


class AddL3PolicyForm(forms.SelfHandlingForm):
    name = forms.CharField(max_length=80, label=_("Name"))
    description = forms.CharField(
        max_length=80, label=_("Description"), required=False)
    ip_version = forms.ChoiceField(choices=[(4, 'IPv4'), (6, 'IPv6')],
                                   widget=forms.Select(attrs={
                                       'class': 'switchable',
                                       'data-slug': 'ipversion',
                                   }),
                                   label=_("IP Version"))
    ip_pool = forms.IPField(label=_("IP Pool"),
                            initial="",
                            help_text=_("Network address in CIDR format "
                                        "(e.g. 192.168.0.0/24,"
                                        "2001:DB8::/48)"),
                            version=forms.IPv4 | forms.IPv6,
                            mask=True)
    subnet_prefix_length = forms.CharField(max_length=80,
                                           label=_("Subnet Prefix Length"),
                                           help_text=_("Between 2 - 30 for IP4"
                                                       "and 2-127 for IP6."),)
    external_segments = \
        fields.CustomMultiChoiceField(label=_("External Segments"),
                                      add_item_link=EXT_SEG_PARAM_URL,
                                      required=False)
    shared = forms.BooleanField(label=_("Shared"),
                                initial=False,
                                required=False)

    def __init__(self, request, *args, **kwargs):
        super(AddL3PolicyForm, self).__init__(request, *args, **kwargs)

    def clean(self):
        cleaned_data = super(AddL3PolicyForm, self).clean()
        if self.is_valid():
            ipversion = int(cleaned_data['ip_version'])
            subnet_prefix_length = int(cleaned_data['subnet_prefix_length'])
            msg = _("Subnet prefix out of range.")
            if ipversion == 4 and subnet_prefix_length not in range(2, 31):
                raise forms.ValidationError(msg)
            if ipversion == 6 and subnet_prefix_length not in range(2, 128):
                raise forms.ValidationError(msg)
        return cleaned_data

    def handle(self, request, context):
        url = reverse("horizon:project:network_policy:index")
        external_segment_dic = {}
        try:
            if context['external_segments']:
                dic = {}
                for external_segment in context['external_segments']:
                    values = [i.split(":")[1]
                        for i in external_segment.split(",")]
                    dic[values[0]] = [values[1]]
                    external_segment_dic.update(dic)
                context['external_segments'] = external_segment_dic
            else:
                context['external_segments'] = {}
            if context.get('name'):
                context['name'] = html.escape(context['name'])
            if context.get('description'):
                context['description'] = html.escape(context['description'])
            client.l3policy_create(request, **context)
            msg = _("L3 Policy Created Successfully!")
            LOG.debug(msg)
            return http.HttpResponseRedirect(url)
        except Exception as e:
            exceptions.handle(request, str(e), redirect=url)


class ExternalSegmentParam(object):

    def __init__(self, context):
        self.external_segment = context['external_segment']
        self.segment_ip = context['segment_ip']
        self.name = "ES:%s,IP:%s" % (
            self.external_segment, self.segment_ip)
        self.id = self.name


class CreateExternalSegmentParamForm(forms.SelfHandlingForm):
    external_segment = forms.ChoiceField(label=_("External Segment"),
                                          required=False)
    segment_ip = forms.IPField(label=_("External Segment IP"), initial="",
                               required=False)

    def __init__(self, request, *args, **kwargs):
        super(CreateExternalSegmentParamForm, self).__init__(request,
                                                             *args,
                                                             **kwargs)
        ec_list = client.externalconnectivity_list(request,
            tenant_id=request.user.tenant_id)
        external_segments_options = [(ec.id, ec.name) for ec in ec_list]
        self.fields['external_segment'].choices = external_segments_options

    def handle(self, request, context):
        return ExternalSegmentParam(context)


class UpdateL3PolicyForm(AddL3PolicyForm):
    def __init__(self, request, *args, **kwargs):
        super(UpdateL3PolicyForm, self).__init__(request, *args, **kwargs)
        try:
            l3policy_id = self.initial['l3policy_id']
            l3 = client.l3policy_get(request, l3policy_id)
            for item in ['name', 'description', 'ip_version',
                         'ip_pool', 'subnet_prefix_length']:
                self.fields[item].initial = str(l3[item])
            self.fields['shared'].initial = l3['shared']
            if bool(l3['external_segments']):
                es_choices = []
                es_initial = []
                for key, value in l3['external_segments'].items():
                    val = 'ES:' + key + ',IP:' + value[0]
                    es_choices.append((val, val))
                    es_initial.append(val)
                self.fields['external_segments'].choices = es_choices
                self.fields['external_segments'].initial = es_initial
        except Exception:
            msg = _("Failed to get L3 policy")
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=shortcuts.redirect)

    def clean(self):
        external_segment_dict = {}
        cleaned_data = super(UpdateL3PolicyForm, self).clean()
        if self.is_valid():
            ipversion = int(cleaned_data['ip_version'])
            subnet_prefix_length = int(cleaned_data['subnet_prefix_length'])
            msg = _("Subnet prefix out of range.")
            if ipversion == 4 and subnet_prefix_length not in range(2, 31):
                raise forms.ValidationError(msg)
            if ipversion == 6 and subnet_prefix_length not in range(2, 128):
                raise forms.ValidationError(msg)
            if cleaned_data['external_segments']:
                dic = {}
                for external_segment in cleaned_data['external_segments']:
                    values = [i.split(":")[1]
                        for i in external_segment.split(",")]
                    dic[values[0]] = [values[1]]
                    external_segment_dict.update(dic)
                cleaned_data['external_segments'] = external_segment_dict
            else:
                cleaned_data['external_segments'] = {}
            updated_data = {d: cleaned_data[d] for d in cleaned_data
                if d in self.changed_data}
            cleaned_data = updated_data
        return cleaned_data

    def handle(self, request, context):
        url = reverse("horizon:project:network_policy:index")
        try:
            l3policy_id = self.initial['l3policy_id']
            if context.get('name'):
                context['name'] = html.escape(context['name'])
            if context.get('description'):
                context['description'] = html.escape(context['description'])
            client.l3policy_update(request, l3policy_id, **context)
            msg = _("L3 Policy Updated Successfully!")
            LOG.debug(msg)
            messages.success(request, msg)
            return http.HttpResponseRedirect(url)
        except Exception as e:
            exceptions.handle(request, str(e), redirect=url)


class AddL2PolicyForm(forms.SelfHandlingForm):
    name = forms.CharField(max_length=80, label=_("Name"), required=False)
    description = forms.CharField(
        max_length=80, label=_("Description"), required=False)
    l3_policy_id = forms.ChoiceField(label=_("L3 Policy"), required=False)
    inject_default_route = forms.BooleanField(label=_("Inject Default Route"),
                                initial=True,
                                required=False)

    def __init__(self, request, *args, **kwargs):
        super(AddL2PolicyForm, self).__init__(request, *args, **kwargs)
        try:
            policies = client.l3policy_list(request,
                tenant_id=request.user.tenant_id)
            policies = [(item['id'], item['name'] + ":" + item['id'])
                        for item in policies]
            self.fields['l3_policy_id'].choices = policies
        except Exception:
            msg = _("Failed to get L3 policy list")
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=shortcuts.redirect)

    def handle(self, request, context):
        url = reverse("horizon:project:network_policy:index")
        try:
            if context.get('name'):
                context['name'] = html.escape(context['name'])
            if context.get('description'):
                context['description'] = html.escape(context['description'])
            client.l2policy_create(request, **context)
            msg = _("L2 Policy Created Successfully!")
            LOG.debug(msg)
            return http.HttpResponseRedirect(url)
        except Exception:
            msg = _("Failed to create L2 policy")
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=shortcuts.redirect)


class UpdateL2PolicyForm(forms.SelfHandlingForm):
    name = forms.CharField(max_length=80, label=_("Name"), required=False)
    description = forms.CharField(
        max_length=80, label=_("Description"), required=False)
    l3_policy_id = forms.ChoiceField(label=_("L3 Policy"), required=False)
    inject_default_route = forms.BooleanField(label=_("Inject Default Route"),
                                required=False)

    def __init__(self, request, *args, **kwargs):
        super(UpdateL2PolicyForm, self).__init__(request, *args, **kwargs)
        try:
            l2policy_id = self.initial['l2policy_id']
            l2 = client.l2policy_get(request, l2policy_id)
            policies = client.l3policy_list(request,
                tenant_id=request.user.tenant_id)
            policies = [(item['id'], item['name'] + ":" + item['id'])
                        for item in policies]
            self.fields['l3_policy_id'].choices = policies
            for item in ['name', 'description', 'l3_policy_id',
                        'inject_default_route']:
                self.fields[item].initial = getattr(l2, item)
        except Exception:
            msg = _("Failed to get L3 policy list")
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=shortcuts.redirect)

    def handle(self, request, context):
        url = reverse("horizon:project:network_policy:l3policy_details",
                      kwargs={'l3policy_id': context['l3_policy_id']})
        l2policy_id = self.initial['l2policy_id']
        try:
            if context.get('name'):
                context['name'] = html.escape(context['name'])
            if context.get('description'):
                context['description'] = html.escape(context['description'])
            client.l2policy_update(request, l2policy_id, **context)
            msg = _("L2 Policy Updated Successfully!")
            LOG.debug(msg)
            return http.HttpResponseRedirect(url)
        except Exception:
            msg = _("Failed to update L2 policy")
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=shortcuts.redirect)


class CreateServicePolicyForm(forms.SelfHandlingForm):
    name = forms.CharField(max_length=80, label=_("Name"))
    description = forms.CharField(
        max_length=80, label=_("Description"), required=False)
    network_service_params = fields.CustomMultiChoiceField(label=_(
        "Network Service Parameters"), add_item_link=NETWORK_PARAM_URL,
        required=False)
    shared = forms.BooleanField(label=_("Shared"),
                                initial=False, required=False)

    def handle(self, request, context):
        url = reverse("horizon:project:network_policy:index")
        try:
            params = context['network_service_params']
            p = []
            if len(params) > 0:
                for item in params:
                    values = [i.split(":")[1] for i in item.split(",")]
                    values = {'type': values[0],
                              'name': values[1],
                              'value': values[2]}
                    p.append(values)
            context['network_service_params'] = p
            if context.get('name'):
                context['name'] = html.escape(context['name'])
            if context.get('description'):
                context['description'] = html.escape(context['description'])
            client.create_networkservice_policy(request, **context)
            msg = _("Service policy created successfully!")
            LOG.debug(msg)
            return http.HttpResponseRedirect(url)
        except Exception as e:
            msg = _("Failed to create service policy. %s") % (str(e))
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=url)


class NetworkServiceParam(object):

    def __init__(self, context):
        self.ptype = context['param_type']
        self.pname = html.escape(context['param_name'])
        self.pvalue = html.escape(context['param_value'])
        self.name = "Type:%s,Name:%s,Value:%s" % (
            self.ptype, self.pname, self.pvalue)
        self.id = self.name


class CreateNetworkServiceParamForm(forms.SelfHandlingForm):
    param_type = forms.ChoiceField(label=_("Type"),
                                   choices=[('ip_single', 'ip_single'),
                                            ('ip_pool', 'ip_pool'),
                                            ('string', 'string')])
    param_name = forms.CharField(max_length=80, label=_("Name"))
    param_value = forms.CharField(max_length=80, label=_("Value"),
            help_text=_("Enter a string. For Types 'ip_single' or 'ip_pool',"
                        "the Value is 'self_subnet' or 'external_subnet'."
                        "For Type 'string' the Value is a user-specified"
                        "string that matches the requirements published"
                        "by a Service Chain Spec."))

    def handle(self, request, context):
        return NetworkServiceParam(context)


class UpdateServicePolicyForm(BaseUpdateForm):
    name = forms.CharField(max_length=80, label=_("Name"))
    description = forms.CharField(
        max_length=80, label=_("Description"), required=False)
    shared = forms.BooleanField(label=_("Shared"), required=False)

    def __init__(self, request, *args, **kwargs):
        super(UpdateServicePolicyForm, self).__init__(request, *args, **kwargs)
        try:
            policy_id = self.initial['service_policy_id']
            policy = client.get_networkservice_policy(request, policy_id)
            self.fields['name'].initial = policy.name
            self.fields['description'].initial = policy.description
            self.fields['shared'].initial = policy.shared
        except Exception as e:
            msg = _("Failed to retrive service policy details. %s") % (str(e))
            LOG.debug(msg)

    def handle(self, request, context):
        url = reverse("horizon:project:network_policy:index")
        try:
            policy_id = self.initial['service_policy_id']
            if context.get('name'):
                context['name'] = html.escape(context['name'])
            if context.get('description'):
                context['description'] = html.escape(context['description'])
            client.update_networkservice_policy(
                request, policy_id, **context)
            msg = _("Service policy updatedsuccessfully!")
            LOG.debug(msg)
            return http.HttpResponseRedirect(url)
        except Exception:
            msg = _("Failed to update service policy")
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=shortcuts.redirect)


class CreateNATPoolForm(forms.SelfHandlingForm):
    name = forms.CharField(max_length=80, label=_("Name"))
    description = forms.CharField(
        max_length=80, label=_("Description"), required=False)
    ip_version = forms.ChoiceField(choices=[(4, 'IPv4'), (6, 'IPv6')],
                                   widget=forms.Select(attrs={
                                       'class': 'switchable',
                                       'data-slug': 'ipversion',
                                   }),
                                   label=_("IP Version"))
    ip_pool = forms.IPField(label=_("CIDR"),
                            initial="", required=True,
                            help_text=_("Network address in CIDR format "
                                        "(e.g. 192.168.0.0/24,"
                                        "2001:DB8::/48)"),
                            version=forms.IPv4 | forms.IPv6, mask=True)
    external_segment_id = forms.ChoiceField(label=_("External Segment"),
                                          required=True)
    shared = forms.BooleanField(label=_("Shared"),
                                initial=False, required=False)

    def __init__(self, request, *args, **kwargs):
        super(CreateNATPoolForm, self).__init__(request,
                                                *args,
                                                **kwargs)
        ec_list = client.externalconnectivity_list(request,
            tenant_id=request.user.tenant_id)
        external_segments_options = [(ec.id, ec.name) for ec in ec_list]
        self.fields['external_segment_id'].choices = external_segments_options

    def handle(self, request, context):
        url = reverse("horizon:project:network_policy:index")
        try:
            if context.get('name'):
                context['name'] = html.escape(context['name'])
            if context.get('description'):
                context['description'] = html.escape(context['description'])
            client.create_natpool(request, **context)
            msg = _("NAT Pool created successfully!")
            LOG.debug(msg)
            messages.success(request, msg)
            return http.HttpResponseRedirect(url)
        except Exception as e:
            msg = _("Failed to NAT Pool. %s") % (str(e))
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=url)


class UpdateNATPoolForm(BaseUpdateForm):
    name = forms.CharField(max_length=80, label=_("Name"))
    description = forms.CharField(
        max_length=80, label=_("Description"), required=False)
    ip_version = forms.ChoiceField(choices=[(4, 'IPv4'), (6, 'IPv6')],
                                   widget=forms.Select(attrs={
                                       'class': 'switchable',
                                       'data-slug': 'ipversion',
                                   }),
                                   label=_("IP Version"))
    ip_pool = forms.IPField(label=_("CIDR"),
                            initial="", required=True,
                            help_text=_("Network address in CIDR format "
                                        "(e.g. 192.168.0.0/24,"
                                        "2001:DB8::/48)"),
                            version=forms.IPv4 | forms.IPv6, mask=True)
    external_segment_id = forms.ChoiceField(label=_("External Segment"),
                                          required=True)
    shared = forms.BooleanField(label=_("Shared"),
                                initial=False, required=False)

    def __init__(self, request, *args, **kwargs):
        super(UpdateNATPoolForm, self).__init__(request,
                                                *args,
                                                **kwargs)
        nat_pool_id = self.initial['nat_pool_id']
        ec_list = client.externalconnectivity_list(request,
            tenant_id=request.user.tenant_id)
        external_segments_options = [(ec.id, ec.name) for ec in ec_list]
        self.fields['external_segment_id'].choices = external_segments_options
        nat_pool = client.get_natpool(request, nat_pool_id)
        attributes = ['name', 'description',
                        'ip_version', 'ip_pool', 'external_segment_id']
        for attr in attributes:
            self.fields[attr].initial = str(nat_pool[attr])
        self.fields['shared'].initial = nat_pool['shared']

    def handle(self, request, context):
        url = reverse("horizon:project:network_policy:index")
        try:
            if context.get('name'):
                context['name'] = html.escape(context['name'])
            if context.get('description'):
                context['description'] = html.escape(context['description'])
            nat_pool_id = self.initial['nat_pool_id']
            client.update_natpool(
                request, nat_pool_id, **context)
            msg = _("NAT Pool updated successfully!")
            LOG.debug(msg)
            messages.success(request, msg)
            return http.HttpResponseRedirect(url)
        except Exception as e:
            msg = _("Failed to update NAT Pool.%s") % \
                (str(e))
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=url)


class UpdateExternalConnectivityForm(forms.SelfHandlingForm):
    name = forms.CharField(max_length=80, label=_("Name"))
    description = forms.CharField(
        max_length=80, label=_("Description"), required=False)
    shared = forms.BooleanField(label=_("Shared"), required=False)

    def __init__(self, request, *args, **kwargs):
        super(UpdateExternalConnectivityForm, self).__init__(request,
            *args, **kwargs)
        try:
            external_connectivity_id = \
                self.initial['external_connectivity_id']
            external_connectivity = client.get_externalconnectivity(request,
                external_connectivity_id)
            self.fields['name'].initial = external_connectivity.name
            self.fields['description'].initial = \
                external_connectivity.description
            self.fields['shared'].initial = external_connectivity.shared
        except Exception as e:
            msg = _("Failed to retrive external connectivity details. %s") % \
                (str(e))
            LOG.debug(msg)

    def handle(self, request, context):
        url = reverse("horizon:project:network_policy:index")
        try:
            external_connectivity_id = self.initial['external_connectivity_id']
            client.update_externalconnectivity(
                request, external_connectivity_id, **context)
            msg = _("External Connectivity updated successfully!")
            LOG.debug(msg)
            return http.HttpResponseRedirect(url)
        except Exception as e:
            msg = _("Failed to update External Connectivity.%s") % \
                (str(e))
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=shortcuts.redirect)


class CreateExternalConnectivityForm(forms.SelfHandlingForm):
    name = forms.CharField(max_length=80, label=_("Name"))
    description = forms.CharField(
        max_length=80, label=_("Description"), required=False)
    ip_version = forms.ChoiceField(choices=[(4, 'IPv4'), (6, 'IPv6')],
                                   widget=forms.Select(attrs={
                                       'class': 'switchable',
                                       'data-slug': 'ipversion',
                                   }),
                                   label=_("IP Version"))
    cidr = forms.IPField(label=_("CIDR"),
                            initial="", required=False,
                            help_text=_("Network address in CIDR format "
                                        "(e.g. 192.168.0.0/24,"
                                        "2001:DB8::/48)"),
                            version=forms.IPv4 | forms.IPv6, mask=True)
    external_routes = fields.CustomMultiChoiceField(
        label=_("External Routes"), add_item_link=ROUTE_URL,
        required=False)
    subnet_id = forms.ChoiceField(label=_("Subnet ID"), required=False)
    port_address_translation = forms.BooleanField(
        label=_("Port Address Translation"),
        initial=False, required=False)
    shared = forms.BooleanField(label=_("Shared"),
                                initial=False, required=False)

    def __init__(self, request, *args, **kwargs):
        super(CreateExternalConnectivityForm, self).__init__(request,
            *args, **kwargs)
        net_id_list = []
        dic = {"router:external": True}
        try:
            net_list = api.neutron.network_list(request, **dic)
            subnet_list = api.neutron.subnet_list(request)
            net_id_list = [net.id for net in net_list]
            self.fields['subnet_id'].choices = [('', 'Select')] + \
                [(subnet.id, subnet.name) for subnet in subnet_list
                if subnet.network_id in net_id_list]
        except Exception:
            msg = _("Failed to get Subnet ID list.")
            exceptions.handle(request, msg)

    def handle(self, request, context):
        try:
            if context['subnet_id'] == '':
                del context['subnet_id']
            if context['cidr'] == '':
                del context['cidr']
            routes = context['external_routes']
            p = []
            if len(routes) > 0:
                for item in routes:
                    values = [i.split(":")[1] for i in item.split(",")]
                    values = {'destination': values[0],
                              'nexthop': values[1]}
                    p.append(values)
            context['external_routes'] = p
            external_segment = client.create_externalconnectivity(
                request, **context)
            msg = _('External Connectivity successfully created.')
            LOG.debug(msg)
            messages.success(request, msg)
            return external_segment
        except Exception as e:
            msg = str(e)
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=shortcuts.redirect)


class ExternalRouteParam(object):

    def __init__(self, context):
        self.destination = context['destination']
        self.next_hop = context['next_hop']
        self.name = 'destination:%s,next_hop:%s' % (
            self.destination, self.next_hop)
        self.id = self.name


class CreateExternalRouteParamForm(forms.SelfHandlingForm):
    destination = forms.IPField(label=_("Destination"),
                            initial="",
                            help_text=_(
                                "(e.g. 192.168.0.0/24,"
                                "2001:DB8::/48)"),
                            version=forms.IPv4 | forms.IPv6,
                            mask=True)
    next_hop = forms.IPField(label=_("Next hop"))

    def handle(self, request, context):
        return ExternalRouteParam(context)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from django.conf.urls import patterns  # noqa
from django.conf.urls import url  # noqa

import views

urlpatterns = patterns('',
                       url(r'^$', views.IndexView.as_view(), name='index'),
                       url(r'^create_service_profile$',
                           views.CreateServiceProfileView.as_view(),
                           name='create_service_profile'),
                       url(r'^serviceprofile/(?P<sp_id>[^/]+)/$',
                           views.ServiceProfileDetailsView.as_view(),
                           name='service_profile_details'),
                       url(r'^create_sc_node$',
                           views.CreateServiceChainNodeView.as_view(),
                           name='create_sc_node'),
                       url(r'^update_sc_node/(?P<scnode_id>[^/]+)/$',
                           views.UpdateServiceChainNodeView.as_view(),
                           name='update_sc_node'),
                       url(r'^sc_node/(?P<scnode_id>[^/]+)/$',
                           views.ServiceChainNodeDetailsView.as_view(),
                           name='sc_node_details'),
                       url(r'^create_sc_spec$',
                           views.CreateServiceChainSpecView.as_view(),
                           name='create_sc_spec'),
                       url(r'^update_sc_spec/(?P<scspec_id>[^/]+)/$',
                           views.UpdateServiceChainSpecView.as_view(),
                           name='update_sc_spec'),
                       url(r'^sc_spec/(?P<scspec_id>[^/]+)/$',
                           views.ServiceChainSpecDetailsView.as_view(),
                           name='sc_spec_details'),
                       url(r'^create_sc_instance$',
                           views.CreateServiceChainInstanceView.as_view(),
                           name='create_sc_instance'),
                       url(r'^update_sc_instance/(?P<scinstance_id>[^/]+)/$',
                           views.UpdateServiceChainInstanceView.as_view(),
                           name='update_sc_instance'),
                       url(r'^sc_instance/(?P<scinstance_id>[^/]+)/$',
                           views.ServiceChainInstanceDetailsView.as_view(),
                           name='sc_instance_details'),
                       )



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from django.conf.urls import patterns  # noqa
from django.conf.urls import url  # noqa

import views

urlpatterns = patterns('',
                       url(r'^$',
                           views.IndexView.as_view(),
                           name='index'),
                       url(r'^addpolicy_target$',
                           views.AddPTGView.as_view(),
                           name='addpolicy_target'),
                       url(r'^addexternal_policy_target$',
                           views.AddExternalPTGView.as_view(),
                           name='addexternal_policy_target'),
                       url(r'^updatepolicy_target/'
                           '(?P<policy_target_id>[^/]+)/$',
                           views.UpdatePTGView.as_view(),
                           name='updatepolicy_target'),
                       url(r'^update_ext_policy_target/'
                          '(?P<ext_policy_target_id>[^/]+)/$',
                           views.UpdateExternalPTGView.as_view(),
                           name='update_ext_policy_target'),
                       url(r'^policy_target/(?P<policy_target_id>[^/]+)/$',
                           views.PTGDetailsView.as_view(),
                           name='policy_targetdetails'),
                       url(r'^ext_policy_target/'
                          '(?P<ext_policy_target_id>[^/]+)/$',
                           views.ExternalPTGDetailsView.as_view(),
                           name='ext_policy_targetdetails'),
                       url(r'^addvm/(?P<policy_target_id>[^/]+)/$',
                           views.LaunchVMView.as_view(), name='addvm'),
                       url(r'^ext_add_policy_rule_set/'
                           '(?P<ext_policy_target_id>[^/]+)/$',
                           views.ExtAddProvidedPRSView.as_view(),
                           name='ext_add_provided_prs'),
                       url(r'^add_policy_rule_set/'
                           '(?P<policy_target_id>[^/]+)/$',
                           views.AddProvidedPRSView.as_view(),
                           name='add_provided_prs'),
                       url(r'^ext_remove_policy_rule_set/'
                           '(?P<ext_policy_target_id>[^/]+)/$',
                           views.ExtRemoveProvidedPRSView.as_view(),
                           name='ext_remove_provided_prs'),
                       url(r'^remove_policy_rule_set/'
                           '(?P<policy_target_id>[^/]+)/$',
                           views.RemoveProvidedPRSView.as_view(),
                           name='remove_provided_prs'),
                       url(r'^add_consumed/(?P<policy_target_id>[^/]+)/$',
                           views.AddConsumedPRSView.as_view(),
                           name='add_consumed_prs'),
                       url(r'^ext_add_consumed/'
                           '(?P<ext_policy_target_id>[^/]+)/$',
                           views.ExtAddConsumedPRSView.as_view(),
                           name='ext_add_consumed_prs'),
                       url(r'^remove_consumed/(?P<policy_target_id>[^/]+)/$',
                           views.RemoveConsumedPRSView.as_view(),
                           name='remove_consumed_prs'),
                       url(r'^ext_remove_consumed/'
                           '(?P<ext_policy_target_id>[^/]+)/$',
                           views.ExtRemoveConsumedPRSView.as_view(),
                           name='ext_remove_consumed_prs'),
                       url(r'/check_ip_availability',
                           views.check_ip_availability,
                           name='check_ip_availability'),
                       )



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import tables


class CreateServiceChainSpecLink(tables.LinkAction):
    name = "create_scspec_link"
    verbose_name = _("Create Service Chain Spec")
    url = "horizon:project:network_services:create_sc_spec"
    classes = ("ajax-modal", "btn-create_scspec")


class EditServiceChainSpecLink(tables.LinkAction):
    name = "edit_sc_spec"
    verbose_name = _("Edit")
    classes = ("ajax-modal", "btn-update",)

    def get_link_url(self, scspec):
        base_url = reverse("horizon:project:network_services:update_sc_spec",
                kwargs={'scspec_id': scspec.id})
        return base_url


class DeleteServiceChainSpecLink(tables.DeleteAction):
    name = "deletescspec"
    action_present = _("Delete")
    action_past = _("Scheduled deletion of %(data_type)s")
    data_type_singular = _("Service Chain Spec")
    data_type_plural = _("Service Chain Specs")


class ServiceChainSpecTable(tables.DataTable):
    name = tables.Column("name",
                     verbose_name=_("Name"),
                     link="horizon:project:network_services:sc_spec_details")
    description = tables.Column("description",
                                verbose_name=_("Description"))
    nodes = tables.Column("nodes", verbose_name=_("Nodes"))

    class Meta(object):
        name = "service_chain_spec_table"
        verbose_name = _("Service Chain Specs")
        table_actions = (CreateServiceChainSpecLink,
                            DeleteServiceChainSpecLink,)
        row_actions = (EditServiceChainSpecLink,
                        DeleteServiceChainSpecLink,)


class CreateServiceChainNodeLink(tables.LinkAction):
    name = "create_scnode_link"
    verbose_name = _("Create Service Chain Node")
    url = "horizon:project:network_services:create_sc_node"
    classes = ("ajax-modal", "btn-create_scnode")


class EditServiceChainNodeLink(tables.LinkAction):
    name = "edit_sc_node"
    verbose_name = _("Edit")
    classes = ("ajax-modal", "btn-update",)

    def get_link_url(self, scnode):
        base_url = reverse("horizon:project:network_services:update_sc_node",
                kwargs={'scnode_id': scnode.id})
        return base_url


class DeleteServiceChainNodeLink(tables.DeleteAction):
    name = "deletescnode"
    action_present = _("Delete")
    action_past = _("Scheduled deletion of %(data_type)s")
    data_type_singular = _("Service Chain Node")
    data_type_plural = _("Service Chain Nodes")


class ServiceChainNodeTable(tables.DataTable):
    name = tables.Column("name",
                     verbose_name=_("Name"),
                     link="horizon:project:network_services:sc_node_details")
    description = tables.Column("description",
                                verbose_name=_("Description"))
    service_profile = tables.Column("service_profile",
                                 verbose_name=_("Service Profile"))

    class Meta(object):
        name = "service_chain_node_table"
        verbose_name = _("Service Chain Node")
        table_actions = (CreateServiceChainNodeLink,
                DeleteServiceChainNodeLink,)
        row_actions = (EditServiceChainNodeLink,
                DeleteServiceChainNodeLink,)


class CreateServiceChainInstanceLink(tables.LinkAction):
    name = "create_scinstance_link"
    verbose_name = _("Create Service Chain Instance")
    url = "horizon:project:network_services:create_sc_instance"
    classes = ("ajax-modal", "btn-create_scinstance")


class EditServiceChainInstanceLink(tables.LinkAction):
    name = "edit_sc_instance"
    verbose_name = _("Edit")
    classes = ("ajax-modal", "btn-update",)

    def get_link_url(self, scinstance):
        urlstring = "horizon:project:network_services:update_sc_instance"
        base_url = reverse(urlstring, kwargs={'scinstance_id': scinstance.id})
        return base_url


class DeleteServiceChainInstanceLink(tables.DeleteAction):
    name = "deletescinstance"
    action_present = _("Delete")
    action_past = _("Scheduled deletion of %(data_type)s")
    data_type_singular = _("ServiceChainInstance")
    data_type_plural = _("ServiceChainInstances")


class ServiceChainInstanceTable(tables.DataTable):
    name = tables.Column("name",
            verbose_name=_("Name"),
            link="horizon:project:network_services:sc_instance_details")
    description = tables.Column("description",
                                verbose_name=_("Description"))
    provider_ptg = tables.Column(
        "provider_ptg", verbose_name=_("Provider"))
    servicechain = tables.Column(
        "servicechain", verbose_name=_("Service Chain"))
    # consumer_ptg = tables.Column(
    #    "consumer_ptg", verbose_name=_("Consumer"))
    servicechain_spec = tables.Column(
        "servicechain_spec", verbose_name=_("Service Chain Spec"))
    classifier = tables.Column("classifier", verbose_name=_("Classifier"))

    class Meta(object):
        name = "service_chain_instance_table"
        verbose_name = _("Service Chain Instance")


class CreateServiceProfileLink(tables.LinkAction):
    name = "create_service_profile_link"
    verbose_name = _("Create Service Profile")
    url = "horizon:project:network_services:create_service_profile"
    classes = ("ajax-modal", "btn-create_service_profile")


class DeleteServiceProfileLink(tables.DeleteAction):
    name = "deleteserviceprofile"
    action_present = _("Delete")
    action_past = _("Scheduled deletion of %(data_type)s")
    data_type_singular = _("ServiceProfile")
    data_type_plural = _("ServiceProfiles")


class ServiceProfileTable(tables.DataTable):
    name = tables.Column("name", verbose_name=_("Name"),
        link="horizon:project:network_services:service_profile_details")
    description = tables.Column("description",
        verbose_name=_("Description"))
    service_type = tables.Column("service_type",
        verbose_name=_("Service Type"))
    insertion_mode = tables.Column("insertion_mode",
        verbose_name=_("Insertion Mode"))
    vendor = tables.Column("vendor", verbose_name=_("Vendor"))

    class Meta(object):
        name = "service_profile_table"
        verbose_name = _("Service Profile")
        table_actions = (CreateServiceProfileLink,
                DeleteServiceProfileLink,)
        row_actions = (DeleteServiceProfileLink,)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import tabs

from gbpui import client
from gbpui import column_filters as gfilters

import tables


class L3PolicyDetailsTab(tabs.Tab):
    name = _("L3 Policy Details")
    slug = "l3_policy_details"
    template_name = "project/endpoint_groups/_l3_policy_details.html"
    failure_url = reverse_lazy('horizon:project:network_policy:index')

    def get_context_data(self, request):
        l3policy_id = self.tab_group.kwargs['l3policy_id']
        try:
            l3policy = client.l3policy_get(request, l3policy_id)
        except Exception:
            exceptions.handle(
                request, _('Unable to retrieve l3 policy details.'),
                redirect=self.failure_url)
        return {'l3policy': l3policy}


class L3PolicyTab(tabs.TableTab):
    table_classes = (tables.L3PolicyTable,)
    name = _("L3 Policy")
    slug = "l3policy"
    template_name = "horizon/common/_detail_table.html"

    def get_l3policy_table_data(self):
        policies = []
        try:
            policies = client.l3policy_list(self.request,
                tenant_id=self.request.user.tenant_id)
            update = lambda x: gfilters.update_l3_policy_attributes(
                self.request, x)
            policies = [update(item) for item in policies]
        except Exception:
            policies = []
            exceptions.handle(self.tab_group.request,
                              _('Unable to retrieve l3 policy list.'))
        return policies


class L2PolicyTab(tabs.TableTab):
    table_classes = (tables.L2PolicyTable,)
    name = _("L2 Policies")
    slug = "l2policy"
    template_name = "horizon/common/_detail_table.html"

    def get_l2policy_table_data(self):
        policies = []
        try:
            policies = client.l2policy_list(self.request,
                tenant_id=self.request.user.tenant_id)
        except Exception:
            policies = []
            exceptions.handle(self.tab_group.request,
                              _('Unable to retrieve l2 policy list.'))
        return policies


class ServicePolicyTab(tabs.TableTab):
    table_classes = (tables.ServicePolicyTable,)
    name = _("Service Policy")
    slug = "service_policy"
    template_name = "horizon/common/_detail_table.html"

    def get_service_policy_table_data(self):
        policies = []
        try:
            policies = client.networkservicepolicy_list(self.request,
                tenant_id=self.request.user.tenant_id)
            update = lambda x: gfilters.update_service_policy_attributes(x)
            policies = [update(item) for item in policies]
        except Exception:
            exceptions.handle(self.tab_group.request,
                    _('Unable to retrieve network service policy list.'))
        return policies


class ServicePolicyDetailsTab(tabs.Tab):
    name = _("Service Policy Details")
    slug = "service_policy_details"
    template_name = "project/network_policy/_service_policy_details.html"
    failure_url = reverse_lazy('horizon:project:network_policy:index')

    def get_context_data(self, request):
        policy_id = self.tab_group.kwargs['service_policy_id']
        try:
            policy = client.get_networkservice_policy(request, policy_id)
        except Exception:
            exceptions.handle(
                request, _('Unable to retrieve service policy details.'),
                redirect=self.failure_url)
        return {'policy': policy}


class ExternalConnectivityTab(tabs.TableTab):
    table_classes = (tables.ExternalConnectivityTable,)
    name = _("External Connectivity")
    slug = "external_connectivity"
    template_name = "horizon/common/_detail_table.html"

    def get_external_connectivity_table_data(self):
        external_segment_list = []
        try:
            external_segment_list = \
                client.externalconnectivity_list(self.request,
                    self.request.user.tenant_id)
        except Exception:
            exceptions.handle(self.tab_group.request,
                    _('Unable to retrieve network service policy list.'))
        return external_segment_list


class ExternalConnectivityDetailsTab(tabs.Tab):
    name = _("External Connectivity Details")
    slug = "external_connectivity_details"
    template_name = \
        "project/network_policy/_external_connectivity_details.html"
    failure_url = reverse_lazy('horizon:project:network_policy:index')

    def get_context_data(self, request):
        external_connectivity_id = \
            self.tab_group.kwargs['external_connectivity_id']
        try:
            external_connectivity = client.get_externalconnectivity(request,
                external_connectivity_id)
        except Exception:
            exceptions.handle(
                request, _('Unable to retrieve service policy details.'),
                redirect=self.failure_url)
        return {'external_connectivity': external_connectivity}


class NATPoolTab(tabs.TableTab):
    table_classes = (tables.NATPoolTable,)
    name = _("NAT Pool")
    slug = "nat_pool"
    template_name = "horizon/common/_detail_table.html"

    def get_nat_pool_table_data(self):
        nat_pool_list = []
        try:
            nat_pools = \
                client.natpool_list(self.request,
                    self.request.user.tenant_id)
            update = lambda x: gfilters.update_nat_pool_attributes(
                self.request, x)
            nat_pool_list = [update(nat_pool) for nat_pool in nat_pools]
        except Exception:
            exceptions.handle(self.tab_group.request,
                    _('Unable to retrieve nat pool list.'))
        return nat_pool_list


class NATPoolDetailsTab(tabs.Tab):
    name = _("NAT Pool Details")
    slug = "nat_pool_details"
    template_name = \
        "project/network_policy/_nat_pool_details.html"
    failure_url = reverse_lazy('horizon:project:network_policy:index')

    def get_context_data(self, request):
        nat_pool_id = \
            self.tab_group.kwargs['nat_pool_id']
        try:
            nat_pool = client.get_natpool(request,
                nat_pool_id)
        except Exception:
            exceptions.handle(
                request, _('Unable to retrieve nat pool details.'),
                redirect=self.failure_url)
        return {'nat_pool': nat_pool}


class ServicePolicyDetailsTabs(tabs.TabGroup):
    slug = "service_policy_details_tab"
    tabs = (ServicePolicyDetailsTab,)
    sticky = True


class ExternalConnectivityDetailsTabs(tabs.TabGroup):
    slug = "external_connectivity_details_tab"
    tabs = (ExternalConnectivityDetailsTab,)
    sticky = True


class NATPoolDetailsTabs(tabs.TabGroup):
    slug = "nat_pool_details_tab"
    tabs = (NATPoolDetailsTab,)
    sticky = True


class L3PolicyTabs(tabs.TabGroup):
    slug = "l3policy_tab"
    tabs = (L3PolicyTab, ServicePolicyTab, ExternalConnectivityTab, NATPoolTab)
    sticky = True


class L2PolicyDetailsTab(tabs.Tab):
    name = _("L2 Policy Details")
    slug = "l2_policy_details"
    template_name = "project/network_policy/_l2_policy_details.html"
    failure_url = reverse_lazy('horizon:project:endpoint_group:index')

    def get_context_data(self, request):
        l2policy_id = self.tab_group.kwargs['l2policy_id']
        try:
            l2policy = client.l2policy_get(request, l2policy_id)
            ptgs = []
            for item in l2policy.policy_target_groups:
                ptgs.append(client.policy_target_get(request, item))
            setattr(l2policy, 'ptgs', ptgs)
        except Exception:
            exceptions.handle(
                request, _('Unable to retrieve l2 policy details.'),
                redirect=self.failure_url)
        return {'l2policy': l2policy}


class L2PolicyDetailsTabs(tabs.TabGroup):
    slug = "l2policy_tabs"
    tabs = (L2PolicyDetailsTab,)


class L3PolicyDetailsTabs(tabs.TabGroup):
    slug = "l3policy_tabs"
    tabs = (L3PolicyDetailsTab, L2PolicyTab,)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from django.core.urlresolvers import reverse
from django import http
from django.utils import html
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages

from gbpui import client

LOG = logging.getLogger(__name__)


class UpdatePolicyTargetForm(forms.SelfHandlingForm):
    name = forms.CharField(max_length=80,
                           label=_("Name"), required=False)
    description = forms.CharField(max_length=80,
                                  label=_("Description"), required=False)
    provided_policy_rule_sets = forms.MultipleChoiceField(
        label=_("Provided Policy Rule Set"), required=False)
    consumed_policy_rule_sets = forms.MultipleChoiceField(
        label=_("Consumed Policy Rule Set"), required=False)
    l2_policy_id = forms.ChoiceField(
        label=_("Network Policy"),
        required=False,
        help_text=_("Select network policy for Group."))
    network_service_policy_id = forms.ChoiceField(
        label=_("Network Services Policy"),
        required=False,
        help_text=_("Select network services policy for Group."))
    shared = forms.BooleanField(label=_("Shared"), required=False)
    failure_url = 'horizon:project:policytargets:index'

    def __init__(self, request, *args, **kwargs):
        super(UpdatePolicyTargetForm, self).__init__(request, *args, **kwargs)
        try:
            policy_target_id = self.initial['policy_target_id']
            policy_target = client.policy_target_get(request, policy_target_id)
            policy_rule_sets = client.policy_rule_set_list(request,
                tenant_id=request.user.tenant_id)
            for c in policy_rule_sets:
                c.set_id_as_name_if_empty()
            policy_rule_sets = sorted(
                policy_rule_sets, key=lambda rule: rule.name)
            policy_rule_set_list = [(c.id, c.name) for c in policy_rule_sets]
            self.fields[
                'provided_policy_rule_sets'].choices = policy_rule_set_list
            self.fields[
                'consumed_policy_rule_sets'].choices = policy_rule_set_list
            provided_init = []
            consumed_init = []
            for item in policy_rule_set_list:
                if item[0] in policy_target.provided_policy_rule_sets:
                    provided_init.append(item[0])
                if item[0] in policy_target.consumed_policy_rule_sets:
                    consumed_init.append(item[0])
            self.fields['provided_policy_rule_sets'].initial = provided_init
            self.fields['consumed_policy_rule_sets'].initial = consumed_init
            n_policies = client.l2policy_list(request,
                tenant_id=request.user.tenant_id)
            ns_policies = client.networkservicepolicy_list(request,
                tenant_id=request.user.tenant_id)
            n_policies = [(item.id, item.name) for item in n_policies]
            ns_policies = [(item.id, item.name) for item in ns_policies]
            ns_policies.insert(0, ('None', 'None'))
            self.fields['l2_policy_id'].choices = n_policies
            self.fields['network_service_policy_id'].choices = ns_policies
            for i in ['name',
                      'description',
                      'l2_policy_id',
                      'network_service_policy_id',
                      'shared']:
                self.fields[i].initial = getattr(policy_target, i)
        except Exception as e:
            msg = _('Unable to retrieve policy_rule_set details. %s') % (
                str(e))
            exceptions.handle(request, msg)
            pass

    def clean(self):
        cleaned_data = super(UpdatePolicyTargetForm, self).clean()
        updated_data = {d: cleaned_data[d] for d in cleaned_data
            if d in self.changed_data}
        return updated_data

    def handle(self, request, context):
        policy_target_id = self.initial['policy_target_id']
        name_or_id = context.get('name') or policy_target_id
        try:
            if 'provided_policy_rule_sets' in context:
                if context.get('provided_policy_rule_sets'):
                    context['provided_policy_rule_sets'] = dict(
                        [(i, 'string')
                        for i in context['provided_policy_rule_sets']])
                else:
                    context['provided_policy_rule_sets'] = None
            if 'consumed_policy_rule_sets' in context:
                if context.get('consumed_policy_rule_sets'):
                    context['consumed_policy_rule_sets'] = dict(
                        [(i, 'string')
                        for i in context['consumed_policy_rule_sets']])
                else:
                    context['consumed_policy_rule_sets'] = None
            if context.get('network_service_policy_id') == 'None':
                context['network_service_policy_id'] = None
            if context.get('name'):
                context['name'] = html.escape(context['name'])
            if context.get('description'):
                context['description'] = html.escape(context['description'])
            policy_target = client.policy_target_update(
                request, policy_target_id, **context)
            msg = _('Group %s was successfully updated.') % name_or_id
            LOG.debug(msg)
            messages.success(request, msg)
            return policy_target
        except Exception as e:
            msg = _('Failed to update Group %(name)s. %(reason)s') % {'name':
                    name_or_id, 'reason': str(e)}
            LOG.error(msg)
            redirect = reverse(self.failure_url)
            exceptions.handle(request, msg, redirect=redirect)


class UpdateExternalPolicyTargetForm(forms.SelfHandlingForm):
    name = forms.CharField(max_length=80,
                           label=_("Name"), required=False)
    description = forms.CharField(max_length=80,
                                  label=_("Description"), required=False)
    provided_policy_rule_sets = forms.MultipleChoiceField(
        label=_("Provided Policy Rule Set"), required=False)
    consumed_policy_rule_sets = forms.MultipleChoiceField(
        label=_("Consumed Policy Rule Set"), required=False)
    external_segments = forms.MultipleChoiceField(
        label=_("External Connectivity"), required=True,
        help_text=_("Select external segment(s) for Group."))
    shared = forms.BooleanField(label=_("Shared"), required=False)
    failure_url = 'horizon:project:policytargets:index'

    def __init__(self, request, *args, **kwargs):
        super(UpdateExternalPolicyTargetForm, self).__init__(request,
            *args, **kwargs)
        try:
            ext_policy_target_id = self.initial['ext_policy_target_id']
            ext_policy_target = client.ext_policy_target_get(request,
                ext_policy_target_id)
            policy_rule_sets = client.policy_rule_set_list(
                request, tenant_id=request.user.tenant_id)
            external_connectivities = client.externalconnectivity_list(
                request, tenant_id=request.user.tenant_id)
            for c in policy_rule_sets:
                c.set_id_as_name_if_empty()
            ext_conn_list = [(e.id, e.name)for e in external_connectivities]
            policy_rule_sets = sorted(
                policy_rule_sets, key=lambda rule: rule.name)
            policy_rule_set_list = [(c.id, c.name) for c in policy_rule_sets]
            self.fields[
                'provided_policy_rule_sets'].choices = policy_rule_set_list
            self.fields[
                'consumed_policy_rule_sets'].choices = policy_rule_set_list
            self.fields[
                'external_segments'].choices = ext_conn_list
            provided_init = []
            consumed_init = []
            for item in policy_rule_set_list:
                if item[0] in ext_policy_target.provided_policy_rule_sets:
                    provided_init.append(item[0])
                if item[0] in ext_policy_target.consumed_policy_rule_sets:
                    consumed_init.append(item[0])
            self.fields['provided_policy_rule_sets'].initial = provided_init
            self.fields['consumed_policy_rule_sets'].initial = consumed_init
            for attr in ['name', 'description', 'shared', 'external_segments']:
                self.fields[attr].initial = getattr(ext_policy_target, attr)
        except Exception as e:
            msg = _('Unable to retrieve policy_rule_set details. %s') % (
                str(e))
            exceptions.handle(request, msg)
            pass

    def clean(self):
        cleaned_data = super(UpdateExternalPolicyTargetForm, self).clean()
        updated_data = {d: cleaned_data[d] for d in cleaned_data
            if d in self.changed_data}
        return updated_data

    def handle(self, request, context):
        ext_policy_target_id = self.initial['ext_policy_target_id']
        name_or_id = context.get('name') or ext_policy_target_id
        try:
            if 'provided_policy_rule_sets' in context:
                if context.get('provided_policy_rule_sets'):
                    context['provided_policy_rule_sets'] = dict(
                        [(i, 'string')
                        for i in context['provided_policy_rule_sets']])
                else:
                    context['provided_policy_rule_sets'] = None
            if 'consumed_policy_rule_sets' in context:
                if context.get('consumed_policy_rule_sets'):
                    context['consumed_policy_rule_sets'] = dict(
                        [(i, 'string')
                        for i in context['consumed_policy_rule_sets']])
                else:
                    context['consumed_policy_rule_sets'] = None
            if context.get('name'):
                context['name'] = html.escape(context['name'])
            if context.get('description'):
                context['description'] = html.escape(context['description'])
            ext_policy_target = client.ext_policy_target_update(
                request, ext_policy_target_id, **context)
            msg = _('External Group %s was successfully updated.') % name_or_id
            LOG.debug(msg)
            messages.success(request, msg)
            return ext_policy_target
        except Exception as e:
            msg = _('Failed to update Group %(name)s. %(reason)s') % {'name':
                    name_or_id, 'reason': str(e)}
            LOG.error(msg)
            redirect = reverse(self.failure_url)
            exceptions.handle(request, msg, redirect=redirect)


class AddProvidedPRSForm(forms.SelfHandlingForm):
    policy_rule_set = forms.MultipleChoiceField(
        label=_("Provided Policy Rule Sets"),)

    def __init__(self, request, *args, **kwargs):
        super(AddProvidedPRSForm, self).__init__(request, *args, **kwargs)
        policy_rule_sets = []
        try:
            policy_target_id = kwargs['initial']['policy_target_id']
            policy_target = client.policy_target_get(request,
                policy_target_id)
            providedpolicy_rule_sets = policy_target.get(
                "provided_policy_rule_sets")
            items = client.policy_rule_set_list(request,
                tenant_id=request.user.tenant_id)
            policy_rule_sets = [
                (p.id, p.name) for p in items
                if p.id not in providedpolicy_rule_sets]
        except Exception as e:
            msg = _('Unable to retrieve policy rule set list.') % (str(e))
            LOG.debug(msg)
        self.fields['policy_rule_set'].choices = policy_rule_sets

    def handle(self, request, context):
        policy_target_id = self.initial['policy_target_id']
        policy_target = client.policy_target_get(request, policy_target_id)
        url = reverse("horizon:project:policytargets:policy_targetdetails",
                      kwargs={'policy_target_id': policy_target_id})
        try:
            for policy_rule_set in policy_target.get(
                    "provided_policy_rule_sets"):
                context['policy_rule_set'].append(policy_rule_set)
            policy_rule_sets = dict([(item, 'string')
                                     for item in context['policy_rule_set']])
            client.policy_target_update(
                request, policy_target_id,
                provided_policy_rule_sets=policy_rule_sets)
            msg = _('Policy Rule Set added successfully!')
            messages.success(request, msg)
            LOG.debug(msg)
            return http.HttpResponseRedirect(url)
        except Exception:
            u = "horizon:project:policytargets:policy_targetdetails"
            msg = _('Failed to add policy_rule_set!')
            redirect = reverse(u, kwargs={'policy_target_id':
                                          policy_target_id})
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=redirect)


class ExtAddProvidedPRSForm(forms.SelfHandlingForm):
    policy_rule_set = forms.MultipleChoiceField(
        label=_("Provided Policy Rule Sets"),)

    def __init__(self, request, *args, **kwargs):
        super(ExtAddProvidedPRSForm, self).__init__(request, *args, **kwargs)
        policy_rule_sets = []
        try:
            ext_policy_target_id = kwargs['initial']['ext_policy_target_id']
            ext_policy_target = client.ext_policy_target_get(request,
                ext_policy_target_id)
            providedpolicy_rule_sets = ext_policy_target.get(
                "provided_policy_rule_sets")
            items = client.policy_rule_set_list(request,
                tenant_id=request.user.tenant_id)
            policy_rule_sets = [
                (p.id, p.name) for p in items
                if p.id not in providedpolicy_rule_sets]
        except Exception as e:
            msg = _('Unable to retrieve policy rule set list.') % (str(e))
            LOG.debug(msg)
        self.fields['policy_rule_set'].choices = policy_rule_sets

    def handle(self, request, context):
        ext_policy_target_id = self.initial['ext_policy_target_id']
        ext_policy_target = client.ext_policy_target_get(request,
            ext_policy_target_id)
        url = reverse("horizon:project:policytargets:ext_policy_targetdetails",
                      kwargs={'ext_policy_target_id': ext_policy_target_id})
        try:
            for policy_rule_set in ext_policy_target.get(
                    "provided_policy_rule_sets"):
                context['policy_rule_set'].append(policy_rule_set)
            policy_rule_sets = dict([(item, 'string')
                                     for item in context['policy_rule_set']])
            client.ext_policy_target_update(
                request, ext_policy_target_id,
                provided_policy_rule_sets=policy_rule_sets)
            msg = _('Policy Rule Set added successfully!')
            messages.success(request, msg)
            LOG.debug(msg)
            return http.HttpResponseRedirect(url)
        except Exception:
            u = "horizon:project:policytargets:ext_policy_targetdetails"
            msg = _('Failed to add policy_rule_set!')
            redirect = reverse(u, kwargs={'ext_policy_target_id':
                                          ext_policy_target_id})
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=redirect)


class RemoveProvidedPRSForm(forms.SelfHandlingForm):
    policy_rule_set = forms.MultipleChoiceField(
        label=_("Provided Policy Rule Sets"),)

    def __init__(self, request, *args, **kwargs):
        super(RemoveProvidedPRSForm, self).__init__(request, *args, **kwargs)
        policy_rule_sets = []
        try:
            policy_target_id = kwargs['initial']['policy_target_id']
            policy_target = client.policy_target_get(request,
                policy_target_id)
            providedpolicy_rule_sets = policy_target.get(
                "provided_policy_rule_sets")
            items = client.policy_rule_set_list(request,
                tenant_id=request.user.tenant_id)
            policy_rule_sets = [(p.id, p.name)
                                for p in items if p.id in
                                providedpolicy_rule_sets]
        except Exception as e:
            msg = _('Unable to retrieve policy rule set list.') % (str(e))
            LOG.debug(msg)
        self.fields['policy_rule_set'].choices = policy_rule_sets

    def handle(self, request, context):
        policy_target_id = self.initial['policy_target_id']
        url = reverse("horizon:project:policytargets:policy_targetdetails",
                      kwargs={'policy_target_id': policy_target_id})
        try:
            policy_target = client.policy_target_get(request, policy_target_id)
            old_policy_rule_sets = policy_target.get(
                "provided_policy_rule_sets")
            for policy_rule_set in context['policy_rule_set']:
                old_policy_rule_sets.remove(policy_rule_set)
            policy_rule_sets = dict([(item, 'string')
                                     for item in old_policy_rule_sets])
            client.policy_target_update(
                request, policy_target_id,
                provided_policy_rule_sets=policy_rule_sets)
            msg = _('Policy Rule Set removed successfully!')
            messages.success(request, msg)
            LOG.debug(msg)
            return http.HttpResponseRedirect(url)
        except Exception:
            msg = _('Failed to remove policy_rule_set!')
            u = "horizon:project:policytargets:policy_targetdetails"
            redirect = reverse(u,
                               kwargs={'policy_target_id': policy_target_id})
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=redirect)


class ExtRemoveProvidedPRSForm(forms.SelfHandlingForm):
    policy_rule_set = forms.MultipleChoiceField(
        label=_("Provided Policy Rule Sets"),)

    def __init__(self, request, *args, **kwargs):
        super(ExtRemoveProvidedPRSForm, self).__init__(request,
            *args, **kwargs)
        policy_rule_sets = []
        try:
            ext_policy_target_id = kwargs['initial']['ext_policy_target_id']
            ext_policy_target = client.ext_policy_target_get(request,
                ext_policy_target_id)
            providedpolicy_rule_sets = ext_policy_target.get(
                "provided_policy_rule_sets")
            items = client.policy_rule_set_list(request,
                tenant_id=request.user.tenant_id)
            policy_rule_sets = [(p.id, p.name)
                                for p in items if p.id in
                                providedpolicy_rule_sets]
        except Exception as e:
            msg = _('Unable to retrieve policy rule set list.') % (str(e))
            LOG.debug(msg)
        self.fields['policy_rule_set'].choices = policy_rule_sets

    def handle(self, request, context):
        ext_policy_target_id = self.initial['ext_policy_target_id']
        url = reverse(
            "horizon:project:policytargets:ext_policy_targetdetails",
            kwargs={'ext_policy_target_id': ext_policy_target_id})
        try:
            ext_policy_target = client.ext_policy_target_get(request,
                ext_policy_target_id)
            old_policy_rule_sets = ext_policy_target.get(
                "provided_policy_rule_sets")
            for policy_rule_set in context['policy_rule_set']:
                old_policy_rule_sets.remove(policy_rule_set)
            policy_rule_sets = dict([(item, 'string')
                                     for item in old_policy_rule_sets])
            client.ext_policy_target_update(
                request, ext_policy_target_id,
                provided_policy_rule_sets=policy_rule_sets)
            msg = _('Policy Rule Set removed successfully!')
            messages.success(request, msg)
            LOG.debug(msg)
            return http.HttpResponseRedirect(url)
        except Exception:
            msg = _('Failed to remove policy_rule_set!')
            u = "horizon:project:policytargets:ext_policy_targetdetails"
            redirect = reverse(u,
                kwargs={'ext_policy_target_id': ext_policy_target_id})
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=redirect)


class AddConsumedPRSForm(forms.SelfHandlingForm):
    policy_rule_set = forms.MultipleChoiceField(
        label=_("Consumed Policy Rule Sets"),)

    def __init__(self, request, *args, **kwargs):
        super(AddConsumedPRSForm, self).__init__(request, *args, **kwargs)
        policy_rule_sets = []
        try:
            policy_target_id = kwargs['initial']['policy_target_id']
            policy_target = client.policy_target_get(request,
                policy_target_id)
            consumedpolicy_rule_sets = policy_target.get(
                "consumed_policy_rule_sets")
            items = client.policy_rule_set_list(request,
                tenant_id=request.user.tenant_id)
            policy_rule_sets = [
                (p.id, p.name) for p in items
                if p.id not in consumedpolicy_rule_sets]
        except Exception:
            pass
        self.fields['policy_rule_set'].choices = policy_rule_sets

    def handle(self, request, context):
        policy_target_id = self.initial['policy_target_id']
        url = reverse("horizon:project:policytargets:policy_targetdetails",
                      kwargs={'policy_target_id': policy_target_id})
        try:
            policy_target = client.policy_target_get(request, policy_target_id)
            for policy_rule_set in policy_target.get(
                    "consumed_policy_rule_sets"):
                context['policy_rule_set'].append(policy_rule_set)
            consumed = dict([(item, 'string')
                             for item in context['policy_rule_set']])
            client.policy_target_update(
                request, policy_target_id, consumed_policy_rule_sets=consumed)
            msg = _('Policy Rule Set Added successfully!')
            messages.success(request, msg)
            LOG.debug(msg)
            return http.HttpResponseRedirect(url)
        except Exception:
            msg = _('Failed to add policy_rule_set!')
            u = "horizon:project:policytargets:policy_targetdetails"
            redirect = reverse(u,
                               kwargs={'policy_target_id': policy_target_id})
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=redirect)


class ExtAddConsumedPRSForm(forms.SelfHandlingForm):
    policy_rule_set = forms.MultipleChoiceField(
        label=_("Consumed Policy Rule Sets"),)

    def __init__(self, request, *args, **kwargs):
        super(ExtAddConsumedPRSForm, self).__init__(request, *args, **kwargs)
        policy_rule_sets = []
        try:
            ext_policy_target_id = kwargs['initial']['ext_policy_target_id']
            ext_policy_target = client.ext_policy_target_get(request,
                ext_policy_target_id)
            consumedpolicy_rule_sets = ext_policy_target.get(
                "consumed_policy_rule_sets")
            items = items = client.policy_rule_set_list(request,
                tenant_id=request.user.tenant_id)
            policy_rule_sets = [
                (p.id, p.name) for p in items
                if p.id not in consumedpolicy_rule_sets]
        except Exception:
            pass
        self.fields['policy_rule_set'].choices = policy_rule_sets

    def handle(self, request, context):
        ext_policy_target_id = self.initial['ext_policy_target_id']
        url = reverse("horizon:project:policytargets:ext_policy_targetdetails",
                      kwargs={'ext_policy_target_id': ext_policy_target_id})
        try:
            ext_policy_target = client.ext_policy_target_get(request,
                ext_policy_target_id)
            for policy_rule_set in ext_policy_target.get(
                    "consumed_policy_rule_sets"):
                context['policy_rule_set'].append(policy_rule_set)
            consumed = dict([(item, 'string')
                             for item in context['policy_rule_set']])
            client.ext_policy_target_update(
                request,
                ext_policy_target_id, consumed_policy_rule_sets=consumed)
            msg = _('Policy Rule Set Added successfully!')
            messages.success(request, msg)
            LOG.debug(msg)
            return http.HttpResponseRedirect(url)
        except Exception:
            msg = _('Failed to add policy_rule_set!')
            u = "horizon:project:policytargets:ext_policy_targetdetails"
            redirect = reverse(u,
                kwargs={'ext_policy_target_id': ext_policy_target_id})
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=redirect)


class RemoveConsumedPRSForm(forms.SelfHandlingForm):
    policy_rule_set = forms.MultipleChoiceField(
        label=_("Consumed Policy Rule Sets"),)

    def __init__(self, request, *args, **kwargs):
        super(RemoveConsumedPRSForm, self).__init__(request, *args, **kwargs)
        policy_rule_sets = []
        try:
            policy_target_id = kwargs['initial']['policy_target_id']
            policy_target = client.policy_target_get(request, policy_target_id)
            consumedpolicy_rule_sets = policy_target.get(
                "consumed_policy_rule_sets")
            items = client.policy_rule_set_list(request,
                tenant_id=request.user.tenant_id)
            policy_rule_sets = [(p.id, p.name)
                                for p in items if p.id
                                in consumedpolicy_rule_sets]
        except Exception:
            pass
        self.fields['policy_rule_set'].choices = policy_rule_sets

    def handle(self, request, context):
        policy_target_id = self.initial['policy_target_id']
        url = reverse("horizon:project:policytargets:policy_targetdetails",
                      kwargs={'policy_target_id': policy_target_id})
        try:
            policy_target = client.policy_target_get(request, policy_target_id)
            old_policy_rule_sets = policy_target.get(
                "consumed_policy_rule_sets")
            for policy_rule_set in context['policy_rule_set']:
                old_policy_rule_sets.remove(policy_rule_set)
            consumed = dict([(item, 'string')
                             for item in old_policy_rule_sets])
            client.policy_target_update(
                request, policy_target_id, consumed_policy_rule_sets=consumed)
            msg = _('Policy Rule Set removed successfully!')
            messages.success(request, msg)
            LOG.debug(msg)
            return http.HttpResponseRedirect(url)
        except Exception:
            msg = _('Failed to remove policy_rule_set!')
            redirect = url
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=redirect)


class ExtRemoveConsumedPRSForm(forms.SelfHandlingForm):
    policy_rule_set = forms.MultipleChoiceField(
        label=_("Consumed Policy Rule Sets"),)

    def __init__(self, request, *args, **kwargs):
        super(ExtRemoveConsumedPRSForm, self).__init__(request,
            *args, **kwargs)
        policy_rule_sets = []
        try:
            ext_policy_target_id = kwargs['initial']['ext_policy_target_id']
            ext_policy_target = client.ext_policy_target_get(request,
                ext_policy_target_id)
            consumedpolicy_rule_sets = ext_policy_target.get(
                "consumed_policy_rule_sets")
            items = client.policy_rule_set_list(request,
                tenant_id=request.user.tenant_id)
            policy_rule_sets = [(p.id, p.name)
                                for p in items if p.id
                                in consumedpolicy_rule_sets]
        except Exception:
            pass
        self.fields['policy_rule_set'].choices = policy_rule_sets

    def handle(self, request, context):
        ext_policy_target_id = self.initial['ext_policy_target_id']
        url = reverse("horizon:project:policytargets:ext_policy_targetdetails",
                      kwargs={'ext_policy_target_id': ext_policy_target_id})
        try:
            ext_policy_target = client.ext_policy_target_get(request,
                ext_policy_target_id)
            old_policy_rule_sets = ext_policy_target.get(
                "consumed_policy_rule_sets")
            for policy_rule_set in context['policy_rule_set']:
                old_policy_rule_sets.remove(policy_rule_set)
            consumed = dict([(item, 'string')
                             for item in old_policy_rule_sets])
            client.ext_policy_target_update(
                request, ext_policy_target_id,
                consumed_policy_rule_sets=consumed)
            msg = _('Policy Rule Set removed successfully!')
            messages.success(request, msg)
            LOG.debug(msg)
            return http.HttpResponseRedirect(url)
        except Exception:
            msg = _('Failed to remove policy_rule_set!')
            redirect = url
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=redirect)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from horizon import tables


class AddAppPolicyLink(tables.LinkAction):
    name = "addpolicy_rule_set"
    verbose_name = _("Create Policy Rule Set")
    url = "horizon:project:application_policy:addpolicy_rule_set"
    classes = ("ajax-modal", "btn-addpolicy_rule_set",)


class UpdateAppPolicyLink(tables.LinkAction):
    name = "updatepolicy_rule_set"
    verbose_name = _("Edit")
    classes = ("ajax-modal", 'edit_policy_rule_set')

    def get_link_url(self, policy_rule_set):
        urlpath = "horizon:project:application_policy:updatepolicy_rule_set"
        base_url = reverse(urlpath, kwargs={
            'policy_rule_set_id': policy_rule_set.id})
        return base_url


class DeleteAppPolicyLink(tables.DeleteAction):
    name = "deletepolicyruleset"
    action_present = _("Delete")
    action_past = _("Scheduled deletion of %(data_type)s")
    data_type_singular = _("Policy Rule Set")
    data_type_plural = _("Policy Rule Sets")


class AddPolicyRuleLink(tables.LinkAction):
    name = "addpolicyrules"
    verbose_name = _("Create Policy Rule")
    url = "horizon:project:application_policy:addpolicyrule"
    classes = ("ajax-modal", "btn-addpolicyrule",)


class UpdatePolicyRuleLink(tables.LinkAction):
    name = "updatepolicyrule"
    verbose_name = _("Edit")
    classes = ("ajax-modal", "btn-update",)

    def get_link_url(self, policy_rule):
        urlstring = "horizon:project:application_policy:updatepolicyrule"
        base_url = reverse(urlstring,
                           kwargs={'policyrule_id': policy_rule.id})
        return base_url


class DeletePolicyRuleLink(tables.DeleteAction):
    name = "deletepolicyrule"
    action_present = _("Delete")
    action_past = _("Scheduled deletion of %(data_type)s")
    data_type_singular = _("Policy Rule")
    data_type_plural = _("Policy Rules")


class AddPolicyClassifierLink(tables.LinkAction):
    name = "addpolicyclassifiers"
    verbose_name = _("Create Policy Classifier")
    url = "horizon:project:application_policy:addpolicyclassifier"
    classes = ("ajax-modal", "btn-addpolicyclassifier",)


class UpdatePolicyClassifierLink(tables.LinkAction):
    name = "updatepolicyclassifier"
    verbose_name = _("Edit")
    classes = ("ajax-modal", "btn-update",)

    def get_link_url(self, policy_classifier):
        base_url = reverse(
            "horizon:project:application_policy:updatepolicyclassifier",
            kwargs={'policyclassifier_id': policy_classifier.id})
        return base_url


class DeletePolicyClassifierLink(tables.DeleteAction):
    name = "deletepolicyclassifier"
    action_present = _("Delete")
    action_past = _("Scheduled deletion of %(data_type)s")
    data_type_singular = _("Policy Classifier")
    data_type_plural = _("Policy Classifiers")


class AddPolicyActionLink(tables.LinkAction):
    name = "addpolicyactions"
    verbose_name = _("Create Policy Action")
    url = "horizon:project:application_policy:addpolicyaction"
    classes = ("ajax-modal", "btn-addpolicyaction",)


class UpdatePolicyActionLink(tables.LinkAction):
    name = "updatepolicyaction"
    verbose_name = _("Edit")
    classes = ("ajax-modal", "btn-update",)

    def get_link_url(self, policy_action):
        urlstring = "horizon:project:application_policy:updatepolicyaction"
        base_url = reverse(urlstring,
                           kwargs={'policyaction_id': policy_action.id})
        return base_url


class DeletePolicyActionLink(tables.DeleteAction):
    name = "deletepolicyaction"
    action_present = _("Delete")
    action_past = _("Scheduled deletion of %(data_type)s")
    data_type_singular = _("Policy Action")
    data_type_plural = _("Policy Actions")


class ApplicationPoliciesTable(tables.DataTable):
    name = tables.Column("name",
            verbose_name=_("Name"),
            link="horizon:project:application_policy:policy_rule_set_details")
    description = tables.Column("description",
                                verbose_name=_("Description"))
    policy_rules = tables.Column("policy_rules",
                                 sortable=False,
                                 verbose_name=_("Policy Rules"))

    class Meta(object):
        name = "application_policies_table"
        verbose_name = _("Policy Rule Set")
        table_actions = (AddAppPolicyLink, DeleteAppPolicyLink)
        row_actions = (UpdateAppPolicyLink, DeleteAppPolicyLink)


class PolicyRulesTable(tables.DataTable):
    name = tables.Column("name",
            verbose_name=_("Name"),
            link="horizon:project:application_policy:policyruledetails")
    description = tables.Column("description",
                                verbose_name=_("Description"))
    enabled = tables.Column("enabled",
                            verbose_name=_("Enabled"))
    policy_classifier = tables.Column("policy_classifier_id",
                                      verbose_name=_("Policy Classifier"))
    policy_actions = tables.Column("policy_actions",
                                      verbose_name=_("Policy Actions"))

    class Meta(object):
        name = "policyrulestable"
        verbose_name = _("Policy Rules")
        table_actions = (AddPolicyRuleLink, DeletePolicyRuleLink)
        row_actions = (UpdatePolicyRuleLink, DeletePolicyRuleLink)


class PolicyClassifiersTable(tables.DataTable):
    name = tables.Column("name",
            verbose_name=_("Name"),
            link="horizon:project:application_policy:policyclassifierdetails")
    description = tables.Column("description",
                                verbose_name=_("Description"))
    protocol = tables.Column("protocol",
                             verbose_name=_("Protocol"))
    port_range = tables.Column("port_range",
                               verbose_name=_("Port Range"))
    direction = tables.Column("direction",
                              verbose_name=_("Direction"))

    class Meta(object):
        name = "policyclassifierstable"
        verbose_name = _("Policy Classifiers")
        table_actions = (AddPolicyClassifierLink, DeletePolicyClassifierLink)
        row_actions = (UpdatePolicyClassifierLink, DeletePolicyClassifierLink)


class PolicyActionsTable(tables.DataTable):
    name = tables.Column("name",
            verbose_name=_("Name"),
            link="horizon:project:application_policy:policyactiondetails")
    description = tables.Column("description",
                                verbose_name=_("Description"))
    action_type = tables.Column("action_type",
                                verbose_name=_("Type"))
    action_value = tables.Column("action_value",
                                 verbose_name=_("Value"))

    class Meta(object):
        name = "policyactionstable"
        verbose_name = _("Policy Actions")
        table_actions = (AddPolicyActionLink, DeletePolicyActionLink)
        row_actions = (UpdatePolicyActionLink, DeletePolicyActionLink)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from django.conf.urls import patterns  # noqa
from django.conf.urls import url  # noqa

import views

urlpatterns = patterns('',
                   url(r'^$', views.IndexView.as_view(), name='index'),
                   url(r'^addl3policy$',
                       views.AddL3policyView.as_view(),
                          name='addl3policy'),
                   url(r'^addservicepolicy$',
                          views.CreateServicePolicyView.as_view(),
                       name='create_servicepolicy'),
                   url(r'^addexternalsegmentparam$',
                          views.AddExternalSegmentParamView.as_view(),
                       name='add_external_segment_param'),
                   url(r'^addnetworkserviceparam$',
                          views.AddNetworkServiceParamView.as_view(),
                       name='add_network_service_param'),
                   url(r'^addexternalrouteparam$',
                          views.AddExternalRouteParamView.as_view(),
                       name='add_external_route_param'),
                   url(r'^createexternalconnectivity$',
                          views.CreateExternalConnectivityView.as_view(),
                       name='create_external_connectivity'),
                   url(r'^createnatpool$',
                          views.CreateNATPoolView.as_view(),
                       name='create_nat_pool'),
                   url(r'^update_servicepolicy/(?P<service_policy_id>[^/]+)/$',
                       views.UpdateServicePolicyView.as_view(),
                       name='update_service_policy'),
                   url(r'^update_externalconnectivity/'
                       '(?P<external_connectivity_id>[^/]+)/$',
                       views.UpdateExternalConnectivityView.as_view(),
                       name='update_externalconnectivity'),
                   url(r'^update_natpool/'
                       '(?P<nat_pool_id>[^/]+)/$',
                       views.UpdateNATPoolView.as_view(),
                       name='update_natpool'),
                   url(r'^natpool/'
                       '(?P<nat_pool_id>[^/]+)/$',
                       views.NATPoolDetailsView.as_view(),
                       name='nat_pool_details'),
                   url(r'^servicepolicy/(?P<service_policy_id>[^/]+)/$',
                       views.ServicePolicyDetailsView.as_view(),
                       name='service_policy_details'),
                   url(r'^externalconnectivity/'
                       '(?P<external_connectivity_id>[^/]+)/$',
                       views.ExternalConnectivityDetailsView.as_view(),
                       name='external_connectivity_details'),
                   url(r'^addl2policy$',
                       views.AddL2policyView.as_view(),
                                               name='addl2policy'),
                   url(r'^l3policy_details/(?P<l3policy_id>[^/]+)/$',
                       views.L3PolicyDetailsView.as_view(),
                                               name='l3policy_details'),
                   url(r'^l3policy_update/(?P<l3policy_id>[^/]+)/$',
                       views.L3PolicyUpdateView.as_view(),
                                               name='update_l3policy'),
                   url(r'^l2policy_details/(?P<l2policy_id>[^/]+)/$',
                       views.L2PolicyDetailsView.as_view(),
                                               name='l2policy_details'),
                   url(r'^l2policy_update/(?P<l2policy_id>[^/]+)/$',
                       views.L2PolicyUpdateView.as_view(),
                                               name='update_l2policy'),
                       )



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from django.core.urlresolvers import reverse
from django import shortcuts
from django.utils import html
from django.utils.text import normalize_newlines  # noqa
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.debug import sensitive_variables  # noqa

from horizon import exceptions
from horizon import forms
from horizon.utils import validators
from horizon import workflows

from openstack_dashboard import api

from openstack_dashboard.dashboards.project.instances \
    import utils as instance_utils
from openstack_dashboard.dashboards.project.instances.workflows \
    import create_instance as workflows_create_instance

from gbpui import client
from gbpui import fields

from netaddr import IPAddress
from netaddr import IPNetwork


LOG = logging.getLogger(__name__)

POLICY_RULE_SET_URL = "horizon:project:application_policy:addpolicy_rule_set"
ADD_EXTERNAL_CONNECTIVITY = \
    "horizon:project:network_policy:create_external_connectivity"


class SelectPolicyRuleSetAction(workflows.Action):
    provided_policy_rule_set = fields.DynamicMultiChoiceField(
        label=_("Provided Policy Rule Set"),
        help_text=_("Choose a policy rule set for an Group."),
        add_item_link=POLICY_RULE_SET_URL,
        required=False)
    consumed_policy_rule_set = fields.DynamicMultiChoiceField(
        label=_("Consumed Policy Rule Set"),
        help_text=_("Select consumed policy rule set for Group."),
        add_item_link=POLICY_RULE_SET_URL,
        required=False)

    class Meta(object):
        name = _("Application Policy")
        help_text = _("Select Policy Rule Set for Group.")

    def _policy_rule_set_list(self, request):
        policy_rule_sets = client.policy_rule_set_list(request,
            tenant_id=request.user.tenant_id)
        for c in policy_rule_sets:
            c.set_id_as_name_if_empty()
        policy_rule_sets = sorted(policy_rule_sets,
                                  key=lambda rule: rule.name)
        return [(c.id, c.name) for c in policy_rule_sets]

    def populate_provided_policy_rule_set_choices(self, request, context):
        policy_rule_set_list = []
        try:
            rsets = self._policy_rule_set_list(request)
            if len(rsets) == 0:
                rsets.extend([('None', 'No Provided Policy Rule Sets')])
            policy_rule_set_list = rsets
        except Exception as e:
            policy_rule_set_list = []
            msg = _('Unable to retrieve policy rule set. %s.') % (str(e))
            exceptions.handle(request, msg)
        return policy_rule_set_list

    def populate_consumed_policy_rule_set_choices(self, request, context):
        policy_rule_set_list = []
        try:
            policy_rule_set_list = [('None', 'No Consumed Policy Rule Sets')]
            policy_rule_set_list =\
                self._policy_rule_set_list(request)
        except Exception as e:
            msg = _('Unable to retrieve policy rule set. %s.') % (str(e))
            exceptions.handle(request, msg)
        return policy_rule_set_list


class SelectL2policyAction(workflows.Action):
    l2policy_id = forms.ChoiceField(
        label=_("Network Policy"),
        help_text=_("Select network policy for Group."))
    network_service_policy_id = forms.ChoiceField(
        label=_("Network Services Policy"),
        help_text=_("Select network services policy for Group."),
        required=False)

    class Meta(object):
        name = _("Network Policy")
        help_text = _(
            "Select network policy for Group."
            "Selecting default will create an Network Policy implicitly.")

    def populate_l2policy_id_choices(self, request, context):
        policies = []
        try:
            policies = client.l2policy_list(request,
                tenant_id=request.user.tenant_id)
            for p in policies:
                p.set_id_as_name_if_empty()
            policies = sorted(policies, key=lambda rule: rule.name)
            policies = [(p.id, p.name + ":" + p.id) for p in policies]
            policies.insert(0, ('default', 'Default'))
        except Exception as e:
            exceptions.handle(request,
                              _("Unable to retrieve policies (%(error)s).")
                              % {'error': str(e)})
        return policies

    def populate_network_service_policy_id_choices(self, request, context):
        policies = []
        try:
            policies = client.networkservicepolicy_list(request,
                tenant_id=request.user.tenant_id)
            for p in policies:
                p.set_id_as_name_if_empty()
            policies = [(p.id, p.name + ":" + p.id) for p in policies]
            policies.insert(0, ('None', 'No Network Service Policy'))
        except Exception as e:
            msg = _("Unable to retrieve service policies. %s).") % (str(e))
            exceptions.handle(request, msg)
        return policies


class SelectL2policyStep(workflows.Step):
    action_class = SelectL2policyAction
    name = _("L2 Policy")
    contributes = ("l2policy_id", "network_services_policy_id",)

    def contribute(self, data, context):
        if data['l2policy_id'] != 'default':
            context['l2_policy_id'] = data['l2policy_id']
        if data['network_service_policy_id'] != 'None':
            context['network_service_policy_id'] = \
                data['network_service_policy_id']
        return context


class SelectPolicyRuleSetStep(workflows.Step):
    action_class = SelectPolicyRuleSetAction
    name = _("Provided Policy Rule Set")
    contributes = ("provided_policy_rule_sets", "consumed_policy_rule_sets",)

    def contribute(self, data, context):
        if data:
            policy_rule_sets = self.workflow.request.POST.getlist(
                "provided_policy_rule_set")
            if policy_rule_sets:
                policy_rule_set_dict = {}
                for policy_rule_set in policy_rule_sets:
                    if policy_rule_set != 'None':
                        policy_rule_set_dict[policy_rule_set] = None
                context['provided_policy_rule_sets'] = policy_rule_set_dict
            policy_rule_sets = self.workflow.request.POST.getlist(
                "consumed_policy_rule_set")
            if policy_rule_sets:
                policy_rule_set_dict = {}
                for policy_rule_set in policy_rule_sets:
                    if policy_rule_set != 'None':
                        policy_rule_set_dict[policy_rule_set] = None
                context['consumed_policy_rule_sets'] = policy_rule_set_dict
            return context


class AddPTGAction(workflows.Action):
    name = forms.CharField(max_length=80,
                           label=_("Name"))
    description = forms.CharField(max_length=80,
                                  label=_("Description"),
                                  required=False)
    shared = forms.BooleanField(label=_("Shared"),
                                initial=False, required=False)

    def __init__(self, request, *args, **kwargs):
        super(AddPTGAction, self).__init__(request, *args, **kwargs)

    class Meta(object):
        name = _("Group")
        help_text = _("Create Internal Group")


class AddPTGStep(workflows.Step):
    action_class = AddPTGAction
    contributes = ("name", "description", "shared")

    def contribute(self, data, context):
        context = super(AddPTGStep, self).contribute(data, context)
        return context


class AddPTG(workflows.Workflow):
    slug = "addpolicy_target"
    name = _("Create Internal Group")
    finalize_button_name = _("Create")
    success_message = _('Create Group "%s".')
    failure_message = _('Unable to create Group "%s".')
    success_url = "horizon:project:policytargets:index"
    default_steps = (AddPTGStep,
                     SelectPolicyRuleSetStep,
                     SelectL2policyStep,)
    wizard = True

    def format_status_message(self, message):
        return message % self.context.get('name')

    def handle(self, request, context):
        try:
            if context.get('name'):
                context['name'] = html.escape(context['name'])
            if context.get('description'):
                context['description'] = html.escape(context['description'])
            group = client.policy_target_create(request, **context)
            return group
        except Exception as e:
            msg = self.format_status_message(self.failure_message) + str(e)
            exceptions.handle(request, msg)
            return False


class ExternalConnectivityAction(workflows.Action):
    external_segments = fields.DynamicMultiChoiceField(
        label=_("External Connectivity"),
        required=True,
        add_item_link=ADD_EXTERNAL_CONNECTIVITY,
        help_text=_("Select external segment(s) for Group."))

    class Meta(object):
        name = _("External Connectivity")
        help_text = _(
            "Select External Connectivity for Group.")

    def populate_external_segments_choices(self, request, context):
        external_connectivities = []
        try:
            external_connectivities = client.externalconnectivity_list(
                request, tenant_id=request.user.tenant_id)
            for p in external_connectivities:
                p.set_id_as_name_if_empty()
            ext_conn_list = sorted(external_connectivities,
                key=lambda segment: segment.name)
            ext_conn_list = \
                [(p.id, p.name + ":" + p.id) for p in ext_conn_list]
        except Exception as e:
            exceptions.handle(request,
                              _("Unable to retrieve policies (%(error)s).")
                              % {'error': str(e)})
        return ext_conn_list


class ExternalConnectivityStep(workflows.Step):
    action_class = ExternalConnectivityAction
    name = _("External Connectivity")
    contributes = ("external_segments",)

    def contribute(self, data, context):
        context['external_segments'] = data.get('external_segments', [])
        return context


class ExtAddPTGAction(workflows.Action):
    name = forms.CharField(max_length=80,
                           label=_("Name"))
    description = forms.CharField(max_length=80,
                                  label=_("Description"),
                                  required=False)
    shared = forms.BooleanField(label=_("Shared"),
                                initial=False, required=False)

    def __init__(self, request, *args, **kwargs):
        super(ExtAddPTGAction, self).__init__(request, *args, **kwargs)

    class Meta(object):
        name = _("Group")
        help_text = _("Create External Group")


class ExtAddPTGStep(workflows.Step):
    action_class = ExtAddPTGAction
    contributes = ("name", "description", "shared")

    def contribute(self, data, context):
        context = super(ExtAddPTGStep, self).contribute(data, context)
        return context


class AddExternalPTG(workflows.Workflow):
    slug = "addexternal_policy_target"
    name = _("Create External Group")
    finalize_button_name = _("Create")
    success_message = _('Create External Group "%s".')
    failure_message = _('Unable to create External Group "%s".')
    success_url = "horizon:project:policytargets:index"
    default_steps = (ExtAddPTGStep,
                     SelectPolicyRuleSetStep,
                     ExternalConnectivityStep,)
    wizard = True

    def format_status_message(self, message):
        return message % self.context.get('name')

    def handle(self, request, context):
        try:
            if context.get('name'):
                context['name'] = html.escape(context['name'])
            if context.get('description'):
                context['description'] = html.escape(context['description'])
            group = client.ext_policy_target_create(request, **context)
            return group
        except Exception as e:
            msg = self.format_status_message(self.failure_message) + str(e)
            exceptions.handle(request, msg)
            return False


KEYPAIR_IMPORT_URL = "horizon:project:access_and_security:keypairs:import"


class SetAccessControlsAction(workflows.Action):
    keypair = forms.DynamicChoiceField(label=_("Key Pair"),
                                       required=False,
                                       help_text=_("Key pair to use for "
                                                   "authentication."),
                                       add_item_link=KEYPAIR_IMPORT_URL)
    admin_pass = forms.RegexField(
        label=_("Admin Password"),
        required=False,
        widget=forms.PasswordInput(render_value=False),
        regex=validators.password_validator(),
        error_messages={'invalid': validators.password_validator_msg()})
    confirm_admin_pass = forms.CharField(
        label=_("Confirm Admin Password"),
        required=False,
        widget=forms.PasswordInput(render_value=False))

    class Meta(object):
        name = _("Access & Security")
        help_text = _("Control access to your instance via key pairs "
                      "and other mechanisms.")

    def __init__(self, request, *args, **kwargs):
        super(SetAccessControlsAction, self).__init__(request, *args, **kwargs)
        self.fields['keypair'].choices = instance_utils.keypair_field_data(
            request, True)
        if not api.nova.can_set_server_password():
            del self.fields['admin_pass']
            del self.fields['confirm_admin_pass']

    def clean(self):
        '''Check to make sure password fields match.'''
        cleaned_data = super(SetAccessControlsAction, self).clean()
        if 'admin_pass' in cleaned_data:
            if cleaned_data['admin_pass'] != cleaned_data.get(
                    'confirm_admin_pass', None):
                raise forms.ValidationError(_('Passwords do not match.'))
        return cleaned_data


class SetAccessControls(workflows.Step):
    action_class = SetAccessControlsAction
    depends_on = ("project_id", "user_id")
    contributes = ("keypair_id", "security_group_ids",
                   "admin_pass", "confirm_admin_pass")

    def contribute(self, data, context):
        if data:
            post = self.workflow.request.POST
            context['security_group_ids'] = post.getlist("groups")
            context['keypair_id'] = data.get("keypair", "")
            context['admin_pass'] = data.get("admin_pass", "")
            context['confirm_admin_pass'] = data.get("confirm_admin_pass", "")
        return context


class SetGroupAction(workflows.Action):
    # To reuse horizon instance launch code related to Networking,
    # form filed must be 'network' only
    network = fields.CustomMultiChoiceField(label=_("Groups"),
                                        widget=forms.CheckboxSelectMultiple(),
                                        error_messages={
                                            'required': _(
                                                "At least one group must"
                                                " be specified.")},
                                        help_text=_("Launch member instance in"
                                                    " these groups"))

    widget = forms.HiddenInput()

    def __init__(self, request, *args, **kwargs):
        super(SetGroupAction, self).__init__(request, *args, **kwargs)
        policy_targetid = self.request.path.split("/")[-2]
        ptg = client.policy_target_get(request, policy_targetid)
        subnet_dedails = None
        for subnet_id in ptg.subnets:
            try:
                subnet = api.neutron.subnet_get(request, subnet_id)
                if subnet_dedails is None:
                    subnet_dedails = subnet['cidr']
                else:
                    subnet_dedails = subnet_dedails + ";" + subnet['cidr']
                allocation_pools = subnet['allocation_pools']
                if allocation_pools:
                    start = allocation_pools[0]['start']
                    end = allocation_pools[0]['end']
                    subnet_dedails = subnet_dedails + "," + start
                    subnet_dedails = subnet_dedails + "," + end
            except Exception as e:
                LOG.error(str(e))
                pass
        initial_value = policy_targetid + ":" + subnet_dedails
        self.fields['network'].initial = [initial_value]

    class Meta(object):
        name = _("Groups")

    def clean(self):
        cleaned_data = super(SetGroupAction, self).clean()
        if not cleaned_data.get("network", None):
            raise forms.ValidationError(_(
                'At least one group must be selected.'))
        return cleaned_data

    def populate_network_choices(self, request, context):
        try:
            pt_list = []
            pts = client.policy_target_list(request,
                tenant_id=request.user.tenant_id)
            for pt in pts:
                pt.set_id_as_name_if_empty()
                subnet_dedails = None
                for subnet_id in pt.subnets:
                    try:
                        subnet = api.neutron.subnet_get(request, subnet_id)
                        if subnet_dedails is None:
                            subnet_dedails = subnet['cidr']
                        else:
                            subnet_dedails += ";" + subnet['cidr']
                        allocation_pools = subnet['allocation_pools']
                        if allocation_pools:
                            start = allocation_pools[0]['start']
                            end = allocation_pools[0]['end']
                            subnet_dedails = subnet_dedails + "," + start
                            subnet_dedails = subnet_dedails + "," + end
                    except Exception as e:
                        LOG.error(str(e))
                        pass
                pt.id = pt.id + ":" + subnet_dedails
                pt_list.append((pt.id, pt.name))
            return sorted(pt_list, key=lambda obj: obj[1])
        except Exception:
            msg = _("Failed to retrieve groups")
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=shortcuts.redirect)


class SetGroup(workflows.Step):
    action_class = SetGroupAction

    template_name = "project/policytargets/_update_groups.html"
    contributes = ("group_id",)

    def contribute(self, data, context):
        if data:
            groups = self.workflow.request.POST.getlist("network")
            groups = [n for n in groups if n != '']
            if groups:
                context['group_id'] = groups

        return context


class LaunchInstance(workflows.Workflow):
    slug = "create_member"
    name = _("Create Member")
    finalize_button_name = _("Launch")
    success_message = _('Launched %(count)s.')
    multipart = True
    default_steps = (workflows_create_instance.SelectProjectUser,
                     workflows_create_instance.SetInstanceDetails,
                     SetAccessControls,
                     SetGroup,
                     workflows_create_instance.PostCreationStep,
                     workflows_create_instance.SetAdvanced)

    def format_status_message(self, message):
        count = self.context.get('count', 1)
        if int(count) > 1:
            return message % {"count": _("%s members") % count}
        else:
            return message % {"count": _("member")}

    def get_success_url(self):
        policy_targetid = self.request.path.split("/")[-2]
        u = "horizon:project:policytargets:policy_targetdetails"
        success_url = reverse(u, kwargs={'policy_target_id': policy_targetid})
        return success_url

    @sensitive_variables('context')
    def handle(self, request, context):
        custom_script = context.get('script_data', '')
        dev_mapping_1 = None
        dev_mapping_2 = None

        image_id = ''

        # Determine volume mapping options
        source_type = context.get('source_type', None)
        if source_type in ['image_id', 'instance_snapshot_id']:
            image_id = context['source_id']
        elif source_type in ['volume_id', 'volume_snapshot_id']:
            try:
                if api.nova.extension_supported("BlockDeviceMappingV2Boot",
                                                request):
                    # Volume source id is extracted from the source
                    volume_source_id = context['source_id'].split(':')[0]
                    device_name = context.get('device_name', '') \
                        .strip() or None
                    dev_source_type_mapping = {
                        'volume_id': 'volume',
                        'volume_snapshot_id': 'snapshot'
                    }
                    dev_mapping_2 = [
                        {'device_name': device_name,
                         'source_type': dev_source_type_mapping[source_type],
                         'destination_type': 'volume',
                         'delete_on_termination':
                             int(bool(context['delete_on_terminate'])),
                         'uuid': volume_source_id,
                         'boot_index': '0',
                         'volume_size': context['volume_size']
                         }
                    ]
                else:
                    dev_mapping_1 = {context['device_name']: '%s::%s' %
                                     (context['source_id'],
                                     int(bool(context['delete_on_terminate'])))
                                     }
            except Exception:
                msg = _('Unable to retrieve extensions information')
                exceptions.handle(request, msg)

        elif source_type == 'volume_image_id':
            device_name = context.get('device_name', '').strip() or None
            dev_mapping_2 = [
                {'device_name': device_name,  # None auto-selects device
                 'source_type': 'image',
                 'destination_type': 'volume',
                 'delete_on_termination':
                     int(bool(context['delete_on_terminate'])),
                 'uuid': context['source_id'],
                 'boot_index': '0',
                 'volume_size': context['volume_size']
                 }
            ]
        avail_zone = context.get('availability_zone', None)
        try:
            instance_count = int(context['count'])
            count = 1
            while count <= instance_count:
                if instance_count == 1:
                    instance_name = context['name']
                else:
                    instance_name = context['name'] + str(count)
                nics = []
                for ptg_id in context['group_id']:
                    values = ptg_id.split(":")
                    ptg_id = values[0]
                    args = {'policy_target_group_id': ptg_id,
                            'name': instance_name[:41] + "_gbpui"}
                    if len(values) == 3:
                        ptg = client.policy_target_get(request, ptg_id)
                        fixed_ip = values[2]
                        for subnet_id in ptg.subnets:
                            subnet = api.neutron.subnet_get(request, subnet_id)
                            if IPAddress(fixed_ip) in \
                                    IPNetwork(subnet['cidr']):
                                args['fixed_ips'] = [
                                    {'subnet_id': subnet['id'],
                                     'ip_address': fixed_ip}]
                                break
                    ep = client.pt_create(request, **args)
                    nics.append({'port-id': ep.port_id})
                api.nova.server_create(request,
                                   instance_name,
                                   image_id,
                                   context['flavor'],
                                   context['keypair_id'],
                                   normalize_newlines(custom_script),
                                   security_groups=None,
                                   block_device_mapping=dev_mapping_1,
                                   block_device_mapping_v2=dev_mapping_2,
                                   nics=nics,
                                   availability_zone=avail_zone,
                                   instance_count=1,
                                   admin_pass=context['admin_pass'],
                                   disk_config=context.get('disk_config'),
                                   config_drive=context.get('config_drive'))
                count += 1
            return True
        except Exception as e:
            error = _("Unable to launch member %(count)s with name %(name)s")
            msg = error % {'count': count, 'name': instance_name}
            LOG.error(str(e))
            u = "horizon:project:policytargets:policy_targetdetails"
            policy_target_id = self.request.path.split("/")[-2]
            redirect = reverse(u, kwargs={'policy_target_id':
                policy_target_id})
            exceptions.handle(request, msg, redirect=redirect)
            return False



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
import logging

from django.core.urlresolvers import reverse
from django.utils import html
from django.utils.translation import ugettext_lazy as _

from django import http
from django import shortcuts

from horizon import exceptions
from horizon import forms
from horizon import messages

from gbpui import client

LOG = logging.getLogger(__name__)

SERVICE_TYPES = [('LOADBALANCER', 'Load Balancer'),
                 ('FIREWALL', 'Firewall')]


class BaseUpdateForm(forms.SelfHandlingForm):

    def clean(self):
        cleaned_data = super(BaseUpdateForm, self).clean()
        updated_data = {d: cleaned_data[d] for d in cleaned_data
            if d in self.changed_data or (d == 'nodes' and
                self.fields['nodes'].initial != cleaned_data[d])}
        return updated_data


class CreateServiceProfileForm(forms.SelfHandlingForm):
    name = forms.CharField(max_length=80, label=_("Name"))
    description = forms.CharField(
        max_length=80, label=_("Description"), required=False)
    vendor = forms.CharField(
        max_length=80, label=_("Vendor"), required=True)
    service_flavor = forms.CharField(
        max_length=80, label=_("Service Flavor"), required=False)
    service_type = forms.ChoiceField(
        label=_("Service Type"), choices=SERVICE_TYPES)
    insertion_mode = forms.ChoiceField(
        label=_("Insertion Mode"), choices=[('l3', 'L3'), ('l2', 'L2')])
    shared = forms.BooleanField(label=_("Shared"),
                                initial=False, required=False)

    def handle(self, request, context):
        url = reverse("horizon:project:network_services:index")
        try:
            if context.get('name'):
                context['name'] = html.escape(context['name'])
            if context.get('description'):
                context['description'] = html.escape(context['description'])
            if context.get('vendor'):
                context['vendor'] = html.escape(context['vendor'])
            if context.get('service_flavor'):
                context['service_flavor'] = html.escape(
                    context['service_flavor'])
            client.create_service_profile(request, **context)
            msg = _("Service Profile  Created Successfully")
            messages.success(request, msg)
            return http.HttpResponseRedirect(url)
        except Exception as e:
            msg = _("Failed to create Service Profile. %s") % (str(e))
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=shortcuts.redirect)


class CreateServiceChainNodeForm(forms.SelfHandlingForm):
    name = forms.CharField(max_length=80, label=_("Name"))
    description = forms.CharField(
        max_length=80, label=_("Description"), required=False)
    service_profile_id = forms.ChoiceField(
        label=_("Service Profile"))
    config_type = forms.ChoiceField(label=_("Config Type"),
                                    choices=[('file', 'Heat Template'),
                                             ('string', 'Config String')],
                                    widget=forms.Select(attrs={'class':
                                                'switchable',
                                                'data-slug': 'source'}))
    template_file = forms.FileField(label=_('Configuration File'),
                        help_text=_('A local Heat template file to upload.'),
                        required=False,
                        widget=forms.FileInput(attrs={'class': 'switched',
                            'data-switch-on': 'source',
                            'data-source-file': _("Configuration File")}))
    template_string = forms.CharField(label=_("Configuration String"),
                          help_text=_('A local Heat template string.'),
                          widget=forms.Textarea(attrs={'class': 'switched',
                                                'data-switch-on': 'source',
                                                'data-source-string':
                                                _("Configuration String")}),
                                            required=False)
    shared = forms.BooleanField(label=_("Shared"),
                                initial=False, required=False)

    def __init__(self, request, *args, **kwargs):
        super(CreateServiceChainNodeForm, self).__init__(
            request, *args, **kwargs)
        try:
            service_profile_tuples = []
            service_profile_list = client.serviceprofile_list(self.request)
            for service_profile in service_profile_list:
                sp = (service_profile.id,
                service_profile.name + ' : ' + service_profile.service_type)
                service_profile_tuples.append(sp)
            self.fields['service_profile_id'].choices = service_profile_tuples
        except Exception:
            msg = _("Failed to retrive Service profile details.")
            LOG.error(msg)

    def clean(self):
        cleaned_data = super(CreateServiceChainNodeForm, self).clean()
        files = self.request.FILES
        template_str = None
        if 'template_file' in files:
            temp = files['template_file'].read()
            try:
                template_str = json.loads(temp)
            except Exception:
                msg = _('Invalid file format.')
                raise forms.ValidationError(msg)
        else:
            try:
                tstr = cleaned_data["template_string"]
                if bool(tstr):
                    template_str = json.loads(tstr)
            except Exception:
                msg = _("Invalid template string.")
                raise forms.ValidationError(msg)
        if template_str is not None:
            cleaned_data['config'] = template_str
        else:
            msg = _("Please choose a template file or enter template string.")
            raise forms.ValidationError(msg)
        return cleaned_data

    def handle(self, request, context):
        url = reverse("horizon:project:network_services:index")
        try:
            try:
                del context['template_string']
                del context['template_file']
                del context['config_type']
            except KeyError:
                pass
            context['config'] = json.dumps(context['config'])
            if context.get('name'):
                context['name'] = html.escape(context['name'])
            if context.get('description'):
                context['description'] = html.escape(context['description'])
            client.create_servicechain_node(request, **context)
            msg = _("Service Chain Node Created Successfully!")
            LOG.debug(msg)
            return http.HttpResponseRedirect(url)
        except Exception as e:
            msg = _("Failed to create Service Chain Node. %s") % (str(e))
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=shortcuts.redirect)


class UpdateServiceChainNodeForm(BaseUpdateForm):
    name = forms.CharField(max_length=80, label=_("Name"))
    description = forms.CharField(
        max_length=80, label=_("Description"), required=False)
    shared = forms.BooleanField(label=_("Shared"), required=False)

    def __init__(self, request, *args, **kwargs):
        super(UpdateServiceChainNodeForm, self).__init__(
            request, *args, **kwargs)
        try:
            scnode_id = self.initial['scnode_id']
            scnode = client.get_servicechain_node(request, scnode_id)
            for item in ['name', 'description', 'shared']:
                self.fields[item].initial = getattr(scnode, item)
        except Exception:
            msg = _("Failed to retrive Service Chain Node details.")
            LOG.error(msg)

    def handle(self, request, context):
        url = reverse("horizon:project:network_services:index")
        try:
            scnode_id = self.initial['scnode_id']
            if context.get('name'):
                context['name'] = html.escape(context['name'])
            if context.get('description'):
                context['description'] = html.escape(context['description'])
            client.update_servicechain_node(
                request, scnode_id, **context)
            msg = _("Service Chain Node Updated Successfully!")
            LOG.debug(msg)
            messages.success(request, msg)
            return http.HttpResponseRedirect(url)
        except Exception as e:
            msg = _("Failed to create Service Chain Node.  %s") % (str(e))
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=url)


class CreateServiceChainSpecForm(forms.SelfHandlingForm):
    name = forms.CharField(max_length=255, label=_("Name"))
    description = forms.CharField(
        max_length=80, label=_("Description"), required=False)
    shared = forms.BooleanField(label=_("Shared"),
                                initial=False, required=False)
    nodes = forms.MultipleChoiceField(label=_("Nodes"),
                                        widget=forms.CheckboxSelectMultiple(),
                                        error_messages={
                                            'required': _(
                                                "At least one node must"
                                                " be specified.")},
                              help_text=_("Create service chain spec with"
                                                    " these nodes"))

    def __init__(self, request, *args, **kwargs):
        super(CreateServiceChainSpecForm, self).__init__(
            request, *args, **kwargs)
        try:
            node_list = []
            nodes = client.servicechainnode_list(request,
                tenant_id=request.user.tenant_id)
            for n in nodes:
                n.set_id_as_name_if_empty()
                node_list.append((n.id, n.name))
            self.fields['nodes'].choices = sorted(
                node_list, key=lambda obj: obj[1])
        except Exception:
            msg = _("Failed to retrieve service chain spec")
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=shortcuts.redirect)

    def handle(self, request, context):
        url = reverse("horizon:project:network_services:index")
        try:
            if context.get('name'):
                context['name'] = html.escape(context['name'])
            if context.get('description'):
                context['description'] = html.escape(context['description'])
            client.create_servicechain_spec(request, **context)
            msg = _("Service Chain Spec Created Successfully!")
            LOG.debug(msg)
            messages.success(request, msg)
            return http.HttpResponseRedirect(url)
        except Exception as e:
            msg = _("Failed to create Service Chain Spec.  %s") % (str(e))
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=url)


class UpdateServiceChainSpecForm(CreateServiceChainSpecForm, BaseUpdateForm):

    def __init__(self, request, *args, **kwargs):
        super(UpdateServiceChainSpecForm, self).__init__(
            request, *args, **kwargs)
        try:
            scspec_id = self.initial['scspec_id']
            scspec = client.get_servicechain_spec(request, scspec_id)
            for attr in ['name', 'description', 'nodes', 'shared']:
                self.fields[attr].initial = getattr(scspec, attr)
            nodes_available = dict(self.fields['nodes'].choices)
            nodes_selected = [
                (x, nodes_available[x]) for x in self.fields['nodes'].initial]
            nodes_unselected = set(self.fields['nodes'].choices) - set(
                nodes_selected)
            self.fields['nodes'].choices = nodes_selected + list(
                nodes_unselected)
        except Exception:
            msg = _("Failed to retrieve service chain spec")
            LOG.error(msg)

    def handle(self, request, context):
        url = reverse("horizon:project:network_services:index")
        try:
            scspec_id = self.initial['scspec_id']
            if context.get('name'):
                context['name'] = html.escape(context['name'])
            if context.get('description'):
                context['description'] = html.escape(context['description'])
            client.update_servicechain_spec(request, scspec_id, **context)
            msg = _("Service Chain Spec Updated Successfully!")
            LOG.debug(msg)
            messages.success(request, msg)
            return http.HttpResponseRedirect(url)
        except Exception as e:
            msg = _("Failed to update Service Chain Spec.  %s") % (str(e))
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=shortcuts.redirect)


class CreateServiceChainInstanceForm(forms.SelfHandlingForm):
    name = forms.CharField(max_length=80, label=_("Name"))
    description = forms.CharField(
        max_length=80, label=_("Description"), required=False)
    servicechain_spec = forms.ChoiceField(label=_("ServiceChain Spec"))
    provider_ptg = forms.ChoiceField(label=_("Provider PTG"))
    consumer_ptg = forms.ChoiceField(label=_("Consumer PTG"))
    classifier = forms.ChoiceField(label=_("Classifier"))

    def __init__(self, request, *args, **kwargs):
        super(CreateServiceChainInstanceForm, self).__init__(
            request, *args, **kwargs)
        try:
            sc_specs = client.servicechainspec_list(request,
                tenant_id=request.user.tenant_id)
            ptgs = client.policy_target_list(request,
                tenant_id=request.user.tenant_id)
            ptgs = [(item.id, item.name) for item in ptgs]
            classifiers = client.policyclassifier_list(request,
                tenant_id=request.user.tenant_id)
            self.fields['servicechain_spec'].choices = [
                (item.id, item.name) for item in sc_specs]
            self.fields['provider_ptg'].choices = ptgs
            self.fields['consumer_ptg'].choices = ptgs
            self.fields['classifier'].choices = [
                (item.id, item.name) for item in classifiers]
        except Exception:
            msg = _("Failed to retrive policy targets")
            LOG.error(msg)

    def handle(self, request, context):
        url = reverse("horizon:project:network_services:index")
        try:
            if context.get('name'):
                context['name'] = html.escape(context['name'])
            if context.get('description'):
                context['description'] = html.escape(context['description'])
            client.create_servicechain_instance(request, **context)
            msg = _("Service Chain Instance Created Successfully!")
            LOG.debug(msg)
            return http.HttpResponseRedirect(url)
        except Exception as e:
            msg = _("Failed to create Service Chain Instance.  %s") % (str(e))
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=shortcuts.redirect)


class UpdateServiceChainInstanceForm(forms.SelfHandlingForm):
    name = forms.CharField(max_length=80, label=_("Name"))
    description = forms.CharField(
        max_length=80, label=_("Description"), required=False)
    servicechain_spec = forms.ChoiceField(label=_("ServiceChain Spec"))

    def __init__(self, request, *args, **kwargs):
        super(UpdateServiceChainInstanceForm, self).__init__(
            request, *args, **kwargs)
        try:
            scinstance_id = self.initial['scinstance_id']
            sc_specs = client.servicechainspec_list(request,
                tenant_id=request.user.tenant_id)
            self.fields['servicechain_spec'].choices = [
                (item.id, item.name) for item in sc_specs]
            scinstance = client.get_servicechain_instance(
                request, scinstance_id)
            for attr in ['name', 'description', 'servicechain_spec']:
                self.fields[attr].initial = getattr(scinstance, attr)
        except Exception:
            pass

    def handle(self, request, context):
        url = reverse("horizon:project:network_services:index")
        try:
            scinstance_id = self.initial['scinstance_id']
            if context.get('name'):
                context['name'] = html.escape(context['name'])
            if context.get('description'):
                context['description'] = html.escape(context['description'])
            client.update_servicechain_instance(
                request, scinstance_id, **context)
            msg = _("Service Chain Instance Created Successfully!")
            LOG.debug(msg)
            return http.HttpResponseRedirect(url)
        except Exception as e:
            msg = _("Failed to create Service Chain Instance.  %s") % (str(e))
            LOG.error(msg)
            exceptions.handle(request, msg, redirect=shortcuts.redirect)
